package com.foreverwin.mesnac.anomaly.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.AbnormalCauseService;
import com.foreverwin.mesnac.anomaly.model.AbnormalCause;
import java.util.List;

/**
 *
 * @author robert
 * @since 2021-07-01
 */
@RestController
@RequestMapping("/Z-ABNORMAL-CAUSE")
public class AbnormalCauseController {

    @Autowired
    public AbnormalCauseService abnormalCauseService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getAbnormalCauseById(@PathVariable String id) {
        return R.ok( abnormalCauseService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getAbnormalCauseList(AbnormalCause abnormalCause){
        List<AbnormalCause> result;
        String site = CommonMethods.getSite();
        abnormalCause.setSite(site);
        QueryWrapper<AbnormalCause> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalCause);
        result = abnormalCauseService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<AbnormalCause> frontPage, AbnormalCause abnormalCause){
        IPage result;
        QueryWrapper<AbnormalCause> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalCause);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(AbnormalCause::getHandle, frontPage.getGlobalQuery())
                .or().like(AbnormalCause::getSite, frontPage.getGlobalQuery())
                .or().like(AbnormalCause::getAbnormalCauseNo, frontPage.getGlobalQuery())
                .or().like(AbnormalCause::getAbnormalBillBo, frontPage.getGlobalQuery())
                .or().like(AbnormalCause::getAbnormalBillDisposeBo, frontPage.getGlobalQuery())
                .or().like(AbnormalCause::getCreatedUser, frontPage.getGlobalQuery())
                .or().like(AbnormalCause::getModifiedUser, frontPage.getGlobalQuery())
    );
        }
        result = abnormalCauseService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param abnormalCause  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody AbnormalCause abnormalCause) {
        return R.ok(abnormalCauseService.save(abnormalCause));
    }

    /**
     * 修改
     * @param abnormalCause  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody AbnormalCause abnormalCause) {
        return R.ok(abnormalCauseService.updateById(abnormalCause));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(abnormalCauseService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(abnormalCauseService.removeByIds(ids));
    }
}