package com.foreverwin.mesnac.anomaly.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.anomaly.dto.AbnormalBillDisposeDto;
import com.foreverwin.mesnac.anomaly.dto.AbnormalBillDto;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalBillDisposeMapper;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalBillMapper;
import com.foreverwin.mesnac.anomaly.model.*;
import com.foreverwin.mesnac.anomaly.service.*;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.dto.ScrapDto;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.helper.NextNumberHelper;
import com.foreverwin.mesnac.common.mapper.MessageMapper;
import com.foreverwin.mesnac.common.model.Message;
import com.foreverwin.mesnac.common.service.AnomalyService;
import com.foreverwin.mesnac.common.service.MessageService;
import com.foreverwin.mesnac.common.util.ActiveMQUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.mapper.*;
import com.foreverwin.mesnac.meapi.model.*;
import com.foreverwin.mesnac.meapi.service.*;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.mesnac.production.mapper.SfcScrapMapper;
import com.foreverwin.mesnac.quality.service.SelfReportService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 异常单 服务实现类
 * </p>
 *
 * @author 赵嘉伟
 * @since 2021-07-22
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class AbnormalBillServiceImpl extends ServiceImpl<AbnormalBillMapper, AbnormalBill> implements AbnormalBillService, AnomalyService {


    @Autowired
    private AbnormalBillMapper abnormalBillMapper;

    @Autowired
    private NextNumberHelper nextNumberHelper;

    @Autowired
    private NcGroupMapper ncGroupMapper;

    @Autowired
    private AbnormalBillDisposeService abnormalBillDisposeService;

    @Autowired
    private ResrceMapper resrceMapper;

    @Autowired
    private NwaUserService nwaUserService;

    @Autowired
    private NcCodeService ncCodeService;

    @Autowired
    private AbnormalNcCodeService abnormalNcCodeService;

    @Autowired
    private AbnormalCauseService abnormalCauseService;

    @Autowired
    private AbnormalDutyService abnormalDutyService;

    @Autowired
    private AbnormalBillDisposeMapper abnormalBillDisposeMapper;

    @Autowired
    private ShopOrderMapper shopOrderMapper;

    @Autowired
    private NwaUserMapper nwaUserMapper;

    @Autowired
    private RouterService routerService;

    @Autowired
    private WorkCenterService workCenterService;

    @Autowired
    private DataFieldListService dataFieldListService;

    @Autowired
    private UploadPicturesService uploadPicturesService;

    @Autowired
    private SfcMapper sfcMapper;

    @Autowired
    private NcCodeMapper ncCodeMapper;

    @Autowired
    private OperationMapper operationMapper;

    @Autowired
    private MessageTypeService messageTypeService;

    @Autowired
    private MessageTypeMapper messageTypeMapper;

    @Autowired
    private WorkCenterMapper workCenterMapper;

    @Autowired
    private ItemMapper itemMapper;

    @Autowired
    private UserGroupMapper userGroupMapper;

    @Autowired
    private ActiveMQUtil activeMQUtil;

    @Autowired
    private MessageMapper messageMapper;

    @Autowired
    private MessageService messageService;

    @Autowired
    private SelfReportService selfReportService;

    @Autowired
    private SfcScrapMapper sfcScrapMapper;

    @Autowired
    private AbnormalPlanService abnormalPlanService;


    @Override
    public IPage<AbnormalBill> selectPage(FrontPage<AbnormalBill> frontPage, AbnormalBill abnormalBill) {
        QueryWrapper<AbnormalBill> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalBill);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<AbnormalBill> selectList(AbnormalBill abnormalBill) {
        QueryWrapper<AbnormalBill> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalBill);
        return super.list(queryWrapper);
    }


    @Override
    public String anomalyReportOther(AbnormalBill abnormalBill,
                                     AbnormalBillDispose abnormalBillDispose,
                                     List<String> dutyCauseType,
                                     List<String> dutyType) {
        String message = this.generateAbnormalBill(abnormalBill, abnormalBillDispose, dutyCauseType, dutyType);
        this.saveOrUpdate(abnormalBill);
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);
        //发送消息
        if(Constants.NEW.equals(abnormalBill.getStatus())){
            this.anomalyCreatedAndSendMessage(abnormalBill,null);
        }
        return message;
    }

    @Override
    public String anomalyReport(AbnormalBill abnormalBill,
                                AbnormalBillDispose abnormalBillDispose,
                                List<String> ncGroupAndNcCodes,
                                List<String> dutyCauseType,
                                List<String> dutyType) {
        String message = this.generateAbnormalBill(abnormalBill, abnormalBillDispose, dutyCauseType, dutyType);
        //获取当前时间
        LocalDateTime now = LocalDateTime.now();
        //生成不良代码和不良代码组,并把它放到hashMap里面，ncCode是存放在异常单里面的ncCode

        List<String> ncCode = new ArrayList<>();
        List<String> ncGroup = new ArrayList<>();
        StringBuilder ncCodes = new StringBuilder();
        for(String ncGroupAndNcCode:ncGroupAndNcCodes){
            String[] split = ncGroupAndNcCode.split("/");
            ncGroup.add(split[0]);
            ncCode.add(split[1]);
            ncCodes.append(split[1]).append(",");
        }

        ArrayList<AbnormalNcCode> abnormalNcCodes = new ArrayList<>();
        for(int i = 0; i < ncGroup.size(); i ++){
            AbnormalNcCode abnormalNcCode = new AbnormalNcCode();
            //设置异常单BO
            abnormalNcCode.setAbnormalBillBo(abnormalBill.getHandle());
            List<AbnormalNcCode> abnormalNcCodes1 = abnormalNcCodeService.selectList(abnormalNcCode);
            if(abnormalNcCodes1.size() != 0){
                QueryWrapper<AbnormalNcCode> abnormalNcCodeQueryWrapper = new QueryWrapper<>();
                abnormalNcCodeQueryWrapper.setEntity(abnormalNcCode);
                abnormalNcCodeService.remove(abnormalNcCodeQueryWrapper);
            }
            abnormalNcCode.setModifiedUser(CommonMethods.getUser());
            abnormalNcCode.setModifiedDateTime(now);
            abnormalNcCode.setCreatedUser(CommonMethods.getUser());
            abnormalNcCode.setCreatedDateTime(now);
            //设置HANDLE
            abnormalNcCode.setHandle(UUID.randomUUID().toString());
            //设置站点
            abnormalNcCode.setSite(CommonMethods.getSite());
            //设置不合格代码组
            abnormalNcCode.setNcCodeGroup(ncGroup.get(i));
            abnormalNcCode.setNcCode(ncCode.get(i));
            abnormalNcCodes.add(abnormalNcCode);
        }
        if(abnormalNcCodes.size() > 0){
            abnormalNcCodeService.saveOrUpdateBatch(abnormalNcCodes);
            abnormalBill.setNcCode(ncCodes.substring(0,ncCodes.length() - 1));
        }
        //判断是否是用户自报
        if(!StringUtil.isBlank(abnormalBill.getInspector())){
            abnormalBill.setObjectBo(HandleEnum.SELF_REPORT.getHandle(abnormalBill.getSite(),abnormalBill.getObjectBo()));
            selfReportService.updateState(abnormalBill.getObjectBo(),Constants.STATE_CONFIRM);
        }


        this.saveOrUpdate(abnormalBill);
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);
        //发送消息
        if(Constants.NEW.equals(abnormalBill.getStatus())){
            this.anomalyCreatedAndSendMessage(abnormalBill,null);
        }

        return message;
    }


    @Override
    public String anomalyReportDevice(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose) {
        String message = this.generateAbnormalBill(abnormalBill, abnormalBillDispose, null, null);
        //判断设备异常是否有为处理完成的sfc
        if(!StringUtil.isBlank(abnormalBill.getSfc()) ){
            List<String> status = new ArrayList<>();
            status.add(Constants.PROGRAM_CONFIRM);
            status.add(Constants.CANCEL);
            status.add(Constants.SHUT_DOWN);
            List<AbnormalBill> abnormalBillList = abnormalBillMapper.findCountBySfc(abnormalBill.getSite(), abnormalBill.getType(), abnormalBill.getSfc(), status);
            if(abnormalBillList.size() >= 1 ){
                throw BusinessException.build("设备异常有未处理完成的sfc");
            }
        }
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);
        this.saveOrUpdate(abnormalBill);
        //发送消息
        this.anomalyCreatedAndSendMessage(abnormalBill,null);
        return message;
    }

    @Override
    public HashMap<String, Object> init(String messageType, String type) {
        HashMap<String, Object> hashMap = new HashMap<>();
        String site = CommonMethods.getSite();
        //消息类型
//        if("Z".equals(type)){
//            List<NcCode> category = ncCodeService.findNcByCategory(messageType,null);
//            hashMap.put("ncByNG",category);
//        }
        //工作中心
        WorkCenter workCenter = new WorkCenter();
        workCenter.setWcCategory("LEVEL4");
        workCenter.setSite(site);
        List<WorkCenter> workCenterList = workCenterService.selectList(workCenter);
        hashMap.put("workCenter",workCenterList);
        //原因分类
        DataField ncCode = new DataField();
        ncCode.setSite(site);
        ncCode.setDataField("REASON_CATEGORY");
        List<DataField> ncCodeList = dataFieldListService.findDataFieldListById(ncCode);
        hashMap.put("causeType",ncCodeList);
        //责任部门
        DataField dataField = new DataField();
        dataField.setSite(site);
        dataField.setDataField("RESPONSIBILITY_DEPARTMENT");
        List<DataField> dutyDepartList = dataFieldListService.findDataFieldListById(dataField);
        hashMap.put("dutyDepart",dutyDepartList);
        //责任分类
        dataField.setDataField("RESPONSIBILITY_TYPE");
        List<DataField> dutyTypeList = dataFieldListService.findDataFieldListById(dataField);
        hashMap.put("dutyType",dutyTypeList);
        //当前登录用户的fullName
        NwaUser nwaUser = new NwaUser();
        nwaUser.setSite(site);
        nwaUser.setUserName(CommonMethods.getUser());
        List<NwaUser> nwaUsers = nwaUserService.selectList(nwaUser);
        if(nwaUsers != null && nwaUsers.size() > 0){
            hashMap.put("currentUser",nwaUsers.get(0).getFullName());
        }
        //初始化异常单号
        hashMap.put("abnormalNo",this.getNextNumber(type));

        return hashMap;
    }

    @Override
    public List<AbnormalBillDto> backLog(AbnormalBillDto abnormalBillDto) {
        String language = LocaleContextHolder.getLocale().getLanguage();
        return abnormalBillMapper.backLog(abnormalBillDto, language);
    }

    @Override
    public AbnormalBillDisposeDto findAllByAbnormalNo(AbnormalBill abnormalBill) {
        String site = CommonMethods.getSite();
        String local=LocaleContextHolder.getLocale().getLanguage();
        abnormalBill.setSite(site);
        if(StringUtil.isEmpty(abnormalBill.getAbnormalNo())){
            return null;
        }
        return abnormalBillDisposeMapper.findAllByAbnormalNo(abnormalBill,local);

    }

    @Override
    public AbnormalBillDto findAllByAbnormalNoDevice(AbnormalBill abnormalBill) {
        String site = CommonMethods.getSite();
        String local=LocaleContextHolder.getLocale().getLanguage();
        abnormalBill.setSite(site);
        if(StringUtil.isEmpty(abnormalBill.getAbnormalNo())){
            return null;
        }
        return abnormalBillMapper.findAllByAbnormalNoDevice(abnormalBill,local);
    }

    @Override
    public void resProcessing(AbnormalBill abnormalBill) {
        String site = CommonMethods.getSite();
        String user = "";
        if(StringUtil.isBlank(abnormalBill.getResponseUser())){
            user = CommonMethods.getUser();
        }else {
            user = abnormalBill.getResponseUser() + "," + CommonMethods.getUser();
        }
        LocalDateTime now = LocalDateTime.now();
        abnormalBill.setSite(site);
        abnormalBill.setResponseUser(user);
        abnormalBill.setResponseDateTime(now);
        abnormalBill.setStatus(Constants.RESPONSE);
        this.saveOrUpdate(abnormalBill);
        //发送消息
        this.anomalyCreatedAndSendMessage(abnormalBillMapper.selectById(abnormalBill.getHandle()),null);
    }

    @Override
    public void programConfirm(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose) {
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        abnormalBillDispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle(site,abnormalBill.getAbnormalNo()));

        if(!StringUtil.isBlank(abnormalBillDispose.getRouterBo())){
            //分割该工艺路线
            String[] routerSplit = abnormalBillDispose.getRouterBo().split(",");
            //判断该返修工艺是否正确
            Router router = new Router();
            router.setSite(site);
            router.setRouter(routerSplit[1]);
            router.setRouterType(routerSplit[2]);
            router.setCurrentRevision("true");
            List<Router> routers = routerService.selectList(router);
            if(routers == null || routers.size() == 0){
                throw BusinessException.build("该返修工艺不存在");
            }
        }
        //判断返修工单是否正确
        if(!StringUtil.isBlank(abnormalBillDispose.getResolveShopOrder())){
            ShopOrder shopOrder = shopOrderMapper.selectById(HandleEnum.SHOP_ORDER.getHandle(site, abnormalBillDispose.getResolveShopOrder()));
            if(shopOrder == null){
                throw BusinessException.build("该返修工单不存在");
            }
        }
        //发送用户
        String[] split = abnormalBillDispose.getResolveSendUser().split(",");
        List<String> strings = Arrays.asList(split);
        int length = nwaUserMapper.checkUser(site, strings);
        if(length != strings.size()){
            throw BusinessException.build("发送用户填报错误");
        }


        //设置解决方案的处理时间和处理人
        abnormalBillDispose.setResolveDateTime(now);
        abnormalBillDispose.setResolveUser(user);
        abnormalBill.setStatus(Constants.PROGRAM_CONFIRM);

        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);
        this.saveOrUpdate(abnormalBill);

        this.anomalyCreatedAndSendMessage(abnormalBillMapper.selectById(abnormalBill.getHandle()),abnormalBillDispose);

    }

    @Override
    public void dutyConfirm(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose) {
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        abnormalBillDispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle(site,abnormalBill.getAbnormalNo()));

        //判断判责提交是否已经填写
        //AbnormalBillDispose abnormalBillDispose1 = abnormalBillDisposeMapper.selectById(abnormalBillDispose.getHandle());
        AbnormalBill abnormalBill1 = abnormalBillMapper.selectById(abnormalBill.getHandle());
        if(!"N".equals(abnormalBill1.getStatus()) && !"X".equals(abnormalBill1.getStatus()) ){
            throw BusinessException.build("该异常单已经判责提交");
        }
        abnormalBill.setStatus(Constants.CORRECTION_CONFIRM);
        abnormalBillDispose.setDutyDateTime(now);
        abnormalBillDispose.setDutyUser(user);

        nwaUserService.checkUserGroup(site,abnormalBillDispose.getDutySendUserGroup());
        this.saveOrUpdate(abnormalBill);
        this.anomalyCreatedAndSendMessage(abnormalBillMapper.selectById(abnormalBill.getHandle()),abnormalBillDispose);
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);

    }

    @Override
    public void anomalyShutDown(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose) {
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        String local = LocaleContextHolder.getLocale().getLanguage();
        abnormalBillDispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle(site,abnormalBill.getAbnormalNo()));

        //判断判责提交是否还没有进行填写
        AbnormalBillDispose abnormalBillDispose1 = abnormalBillDisposeMapper.selectById(abnormalBillDispose.getHandle());
        if(abnormalBillDispose1 != null){
            if(StringUtil.isBlank(abnormalBillDispose1.getDutyCauseType())){
                throw BusinessException.build("该异常单还没有进行判责提交");
            }
        }



        abnormalBill.setStatus(Constants.SHUT_DOWN);
        abnormalBillDispose.setClosedUser(user);
        abnormalBillDispose.setClosedDateTime(now);
       // this.anomalyCreatedAndSendMessage(abnormalBill,abnormalBillDispose);
        //保存到异常计划表
        AbnormalBillDisposeDto abnormalBillDisposeDto = abnormalBillDisposeMapper.findAllByAbnormalNo(abnormalBill, local);
        abnormalPlanService.saveAbnormalPlan(abnormalBillDisposeDto,now);
        this.saveOrUpdate(abnormalBill);
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);

    }


    @Override
    public String generateAbnormalBill(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose,List<String> dutyCauseType,List<String> dutyType) {
        String site = CommonMethods.getSite();
        //返回的消息
        String message = "提报成功";
        LocalDateTime currentTime = LocalDateTime.now();
        String user = CommonMethods.getUser();
        //如果前台传过来的异常单为空的话，则抛出一个异常
        if(StringUtil.isBlank(abnormalBill.getAbnormalNo())) {
            throw BusinessException.build("异常单号不存在");
        }
        //设置异常单的handle
        abnormalBill.setHandle(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalBill.getAbnormalNo()));
        if(abnormalBillMapper.selectById(abnormalBill.getHandle()) != null &&
                !Constants.SHUT_DOWN.equals(abnormalBill.getStatus())){
            throw BusinessException.build("该异常单已经提交，请清空后提交");
        }
        /**
         * 取消异常提报的更新
         */

        //更新的时候，判断该异常单的状态是否为新建，如果为新建，则可以修改，否则就不能进行修改
//        AbnormalBill abnormalBill1 = abnormalBillMapper.selectById(abnormalBill.getHandle());
//        if(abnormalBill1 != null){
//            if(!"N".equals(abnormalBill1.getStatus()) && StringUtil.isBlank(abnormalBillDispose.getClosedUser())){
//                throw BusinessException.build("该异常单已经被人处理，不能进行更新");
//            }
//            message = "更新成功";
//        }else{
//            message = "保存成功";
//        }
//        abnormalBillDispose.setClosedUser(null);
        //设置异常单详细表的handle和abnormalBillBo
        abnormalBillDispose.setAbnormalBillBo(abnormalBill.getHandle());
        abnormalBillDispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle(site,abnormalBill.getAbnormalNo()));

        //设置责任划分填报人和填报时间
        abnormalBillDispose.setDutyUser(user);
        abnormalBillDispose.setDutyDateTime(currentTime);

        //判断原因分类是否为空,如果不为空，把它放进Z_ABNORMAL_CAUSE表里面
        ArrayList<AbnormalCause> causeTypes = new ArrayList<>();
        if(dutyCauseType != null && dutyCauseType.size() >= 1){
            for(String cause:dutyCauseType){
                AbnormalCause abnormalCause = new AbnormalCause();
                String uuid = UUID.randomUUID().toString();
                abnormalCause.setSite(site);
                abnormalCause.setAbnormalBillBo(abnormalBill.getHandle());
                abnormalCause.setAbnormalBillDisposeBo(abnormalBillDispose.getHandle());
                List<AbnormalCause> abnormalCauses = abnormalCauseService.selectList(abnormalCause);
                if(abnormalCauses.size() != 0){
                    QueryWrapper<AbnormalCause> abnormalCauseQueryWrapper = new QueryWrapper<>();
                    abnormalCauseQueryWrapper.setEntity(abnormalCause);
                    abnormalCauseService.remove(abnormalCauseQueryWrapper);
                }
                abnormalCause.setHandle(uuid);
                abnormalCause.setAbnormalCauseNo(cause);
                abnormalCause.setCreatedDateTime(currentTime);
                abnormalCause.setCreatedUser(user);
                abnormalCause.setModifiedUser(user);
                abnormalCause.setModifiedDateTime(currentTime);
                causeTypes.add(abnormalCause);
            }
        }

        //判断工序编码是否传递错误
        if(!StringUtil.isBlank(abnormalBill.getOperation())){
            QueryWrapper<Operation> operationQueryWrapper = new QueryWrapper<Operation>();
            operationQueryWrapper.eq("OPERATION",abnormalBill.getOperation())
                    .eq("SITE",site);
            List<Operation> operations = operationMapper.selectList(operationQueryWrapper);
            if(operations == null || operations.size() == 0){
                throw BusinessException.build("工序编号传递错误");
            }
        }

        //判断设备传递的不良代码是否有误
        if(!StringUtil.isBlank(abnormalBill.getNcCode()) && Constants.ABNORMAL_RESRCE.equals(abnormalBill.getType())){
            List<String> ncCodes = Arrays.asList(abnormalBill.getNcCode().split(","));
            Integer length = ncCodeMapper.checkNcCode(ncCodes,site,"FAILURE");
            if(ncCodes.size() != length){
                throw BusinessException.build("不良代码传递有误");
            }
        }


        ArrayList<AbnormalDuty> dutyTypes = new ArrayList<>();
        //判断责任分类是否为空,如果不为空，把它放进Z_ABNORMAL_DUTY表里面
        if(dutyType != null && dutyType.size() >= 1){
            for(String duty:dutyType){
                AbnormalDuty abnormalDuty = new AbnormalDuty();
                String uuid = UUID.randomUUID().toString();
                abnormalDuty.setSite(site);
                abnormalDuty.setAbnormalBillBo(abnormalBill.getHandle());
                abnormalDuty.setAbnormalBillDisposeBo(abnormalBillDispose.getHandle());
                List<AbnormalDuty> abnormalDuties = abnormalDutyService.selectList(abnormalDuty);
                if(abnormalDuties.size() != 0){
                    QueryWrapper<AbnormalDuty> abnormalDutyQueryWrapper = new QueryWrapper<>();
                    abnormalDutyQueryWrapper.setEntity(abnormalDuty);
                    abnormalDutyService.remove(abnormalDutyQueryWrapper);
                }
                abnormalDuty.setHandle(uuid);
                abnormalDuty.setAbnormalDutyNo(duty);
                abnormalDuty.setModifiedUser(user);
                abnormalDuty.setModifiedDateTime(currentTime);
                abnormalDuty.setCreatedDateTime(currentTime);
                abnormalDuty.setCreatedUser(user);
                dutyTypes.add(abnormalDuty);
            }
        }

        //设置创建时间和更新时间
        if(abnormalBillMapper.selectById(abnormalBill.getHandle()) == null){
            abnormalBill.setCreatedDateTime(currentTime);
            abnormalBill.setCreatedUser(user);
            abnormalBillDispose.setCreatedDataTime(currentTime);
            abnormalBillDispose.setCreatedUser(user);
        }
        abnormalBill.setModifiedDateTime(currentTime);
        abnormalBill.setModifiedUser(user);
        abnormalBillDispose.setModifiedDateTime(currentTime);
        abnormalBillDispose.setModifiedUser(user);

        /**
         * 判断SFC ，设备异常提交，sfc可以为空
         * 设备编号
         * 是否为空
         */
        if(StringUtil.isBlank(abnormalBill.getSfc()) && !Constants.ABNORMAL_RESRCE.equals(abnormalBill.getType())) {
            throw BusinessException.build("产品条码不能为空");
        }
        //异常提报的时候，提报用户不可以为空
        if(StringUtil.isBlank(abnormalBill.getReportSendUserGroup()) ) {
            throw BusinessException.build("提报用户组不能为空");
        }
//        }else if(!StringUtil.isBlank(abnormalBill.getSfc()) && Constants.ABNORMAL_RESRCE.equals(abnormalBill.getType())){
//            AbnormalBill abnormalBill2 = new AbnormalBill();
//            abnormalBill2.setType(abnormalBill.getType());
//            abnormalBill2.setSfc(abnormalBill.getSfc());
//            abnormalBill2.setStatus(Constants.NEW);
//            List<AbnormalBill> abnormalBills = this.selectList(abnormalBill2);
//            if(abnormalBills.size() > 0 && !"更新成功".equals(message)){
//                throw BusinessException.build("设备异常有未处理完成的sfc");
//            }else if(abnormalBills.size() > 1){
//                throw BusinessException.build("设备异常有未处理完成的sfc");
//            }
//        }
//        if(StringUtil.isBlank(abnormalBill.getResrce())){
//            throw BusinessException.build("设备编号不能为空");
//        }
//        if(!StringUtil.isBlank(abnormalBill.getMessageType())){
//            NcGroup messageType = ncGroupMapper.selectById(HandleEnum.NC_GROUP.getHandle(site, abnormalBill.getMessageType()));
//            if(messageType == null){
//                throw BusinessException.build("该消息类型不存在");
//            }
//        }

        if(!StringUtil.isBlank(abnormalBill.getResrce()) && Constants.ABNORMAL_RESRCE.equals(abnormalBill.getType())){
            Resrce resrce = resrceMapper.selectById(HandleEnum.RESOURCE.getHandle(site, abnormalBill.getResrce()));
            if(resrce == null){
                throw BusinessException.build("该设备编号不存在");
            }
        }
        //判断不良品数量和问题数量是否为正整数
        try {
            if(!StringUtil.isBlank(abnormalBill.getNcQty()) && Integer.parseInt(abnormalBill.getNcQty()) < 0) {
                throw BusinessException.build("不良品数量必须大于0");
            }
            if(!StringUtil.isBlank(abnormalBill.getPbQty()) && Integer.parseInt(abnormalBill.getPbQty()) < 0){
                throw BusinessException.build("问题数量必须大于0");
            }
        }catch (NumberFormatException num){
            throw BusinessException.build("问题数量或不良品数量必须为一个正整数");
        }

        //判断输入的责任划分用户组是否正确，并返回对应的用户
        nwaUserService.checkUserGroup(site,abnormalBillDispose.getDutySendUserGroup());
        //插入原因分类和责任分类
        if(dutyTypes.size() > 0){
            abnormalDutyService.saveOrUpdateBatch(dutyTypes);
        }
        if(causeTypes.size() > 0){
            abnormalCauseService.saveOrUpdateBatch(causeTypes);
        }
        if(!StringUtil.isBlank(abnormalBill.getFileName())){
            //放置图片
            this.fileUpload(abnormalBill);
        }

//        this.anomalyCreatedAndSendMessage(abnormalBill,null);
        return message;

    }

    @Override
    public boolean cancelBrowse(String handle, String cancelBrowse,String code,String cancelSendUserGroup) {
        LocalDateTime now = LocalDateTime.now();
        AbnormalBill abnormalBill = abnormalBillMapper.selectById(handle);
        if(abnormalBill == null){
            return  false;
        }
        abnormalBill.setStatus(Constants.CANCEL);
        abnormalBill.setCancelReason(cancelBrowse);
        abnormalBill.setCancelSendUserGroup(cancelSendUserGroup);
        if(!StringUtil.isBlank(code)){
            abnormalBill.setCancelCode(code);
        }
        abnormalBill.setCancelUser(CommonMethods.getUser());
        abnormalBill.setCancelDateTime(now);
        this.anomalyCreatedAndSendMessage(abnormalBill,null);
        this.saveOrUpdate(abnormalBill);
        return  true;
    }

    @Override
    public void fileUpload(AbnormalBill abnormalBill){
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        List<String> name = null;
        if(StringUtil.notBlank(abnormalBill.getFileName())){
            name = Arrays.asList(abnormalBill.getFileName().split(","));
        }
        if(name != null && name.size() != 0){
            //String uuid = UUID.randomUUID().toString();
            UploadPictures uploadPictures = new UploadPictures();
            uploadPictures.setHandle("UploadPicturesBO:"+abnormalBill.getSite()+","+abnormalBill.getAbnormalNo());
            uploadPictures.setSite(abnormalBill.getSite());
            uploadPictures.setPicturePath(abnormalBill.getFilePath());
            StringBuilder pictureAddress = new StringBuilder();
            for (int i = 0; i <= name.size() -1; i++) {
                if(i != name.size() -1){
                    pictureAddress.append(abnormalBill.getFilePath()).append("/").append(name.get(i)).append(",");
                }else{
                    pictureAddress.append(abnormalBill.getFilePath()).append("/").append(name.get(i));
                }
            }
            uploadPictures.setPictureAddress(pictureAddress.toString());
            uploadPictures.setObjectBo(abnormalBill.getHandle());
            uploadPictures.setNum(abnormalBill.getFileNum());
            uploadPictures.setType("ABNORMAL");
            uploadPictures.setCreatedUser(user);
            uploadPictures.setCreatedDateTime(now);
            uploadPictures.setModifiedUser(user);
            uploadPictures.setModifiedDateTime(now);
            uploadPicturesService.saveOrUpdate(uploadPictures);
        }
    }

    @Override
    public UploadPictures findUploadPictureByAbnormalBo(AbnormalBill abnormalBill) {
        String site = CommonMethods.getSite();
        UploadPictures uploadPictures = new UploadPictures();
        uploadPictures.setSite(site);
        uploadPictures.setObjectBo(abnormalBill.getHandle());
        List<UploadPictures> uploadPictures1 = uploadPicturesService.selectList(uploadPictures);
        if(uploadPictures1 != null && uploadPictures1.size() == 1){
            return  uploadPictures1.get(0);
        }
        return null;
    }

    @Override
    public String getNextNumber(String type){
        String site = CommonMethods.getSite();
        String abnormalNo = "";
        //生成异常单号
        if("Q".equals(type)){//其他异常
            abnormalNo = nextNumberHelper.getNextNumber(site, "ABNORMAL_OTHER", 1, null).get(0);
        }else if("Z".equals(type)){//质量异常
            abnormalNo = nextNumberHelper.getNextNumber(site, "ABNORMAL_QUALITY", 1, null).get(0);
        }else if("S".equals(type)){//设备异常
            abnormalNo = nextNumberHelper.getNextNumber(site, "ABNORMAL_DEVICE", 1, null).get(0);
        }
        return abnormalNo;
    }

    @Override
    public HashMap<String,Object> anomalyReveseRepair(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose) {
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        if(!Constants.RESPONSE.equals(abnormalBill.getStatus())){
            throw BusinessException.build("该异常单的状态必须是响应中才能转维修");
        }

        if(StringUtil.isBlank(abnormalBillDispose.getAbnormalMethod())){
            throw BusinessException.build("转维修时方案分类必须填写");
        }

        abnormalBill.setHandle(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalBill.getAbnormalNo()));
        abnormalBill.setStatus(Constants.SHUT_DOWN);
        abnormalBillDispose.setResolveUser(user);
        abnormalBillDispose.setResolveDateTime(now);
        abnormalBillDispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle(site,abnormalBill.getAbnormalNo()));
        this.saveOrUpdate(abnormalBill);
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDispose);
        HashMap<String, Object> hashMap = new HashMap<>();
        hashMap.put("abnormalBill",abnormalBillMapper.selectById(abnormalBill.getHandle()));
        hashMap.put("abnormalBillDispose",abnormalBillDisposeMapper.selectById(abnormalBillDispose.getHandle()));
        return hashMap;
    }

    @Override
    public void createAnomalyTask(String category, String resource, String shopOrder,
                                  String sfc, String ncCodes,String position,String ncQty,
                                  String ncDescription,String userGroup) {
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        String user = CommonMethods.getUser();
        String abnormalNo = null;

        AbnormalBill abnormalBill = new AbnormalBill();
        if(Constants.ABNORMAL_RESRCE.equals(category)){
            abnormalNo = this.getNextNumber(category);
            abnormalBill.setNcCode(ncCodes);
            abnormalBill.setRepairDateTime(now);
            abnormalBill.setType(category);
        }else if(Constants.ABNORMAL_OTHER.equals(category)){
            abnormalNo = this.getNextNumber(category);
            abnormalBill.setType(category);
        }

        abnormalBill.setSite(site);
        abnormalBill.setAbnormalNo(abnormalNo);
        abnormalBill.setHandle(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalNo));
        abnormalBill.setStatus(Constants.NEW);
        abnormalBill.setSfc(sfc);
        abnormalBill.setShopOrder(shopOrder);
        //根据工单找itemBo，并放到异常单里面
        if(shopOrder != null ){
            ShopOrder shopOrder1 = shopOrderMapper.selectById(HandleEnum.SHOP_ORDER.getHandle(site, shopOrder));
            abnormalBill.setItemBo(shopOrder1.getItemBo());
        }
        abnormalBill.setResrce(resource);
        abnormalBill.setEntityLocation(position);
        abnormalBill.setNcQty(ncQty);
        abnormalBill.setPbDescription(ncDescription);
        abnormalBill.setDiscover(Constants.SELF_MADE);
        abnormalBill.setReportFrom(Constants.REPORT);
        abnormalBill.setReportSendUserGroup(userGroup);
        AbnormalBillDispose abnormalBillDispose = new AbnormalBillDispose();

        abnormalBillDispose.setSite(site);
        abnormalBillDispose.setAbnormalBillBo(abnormalBill.getHandle());


        //判断工单，sfc，资源是否传递正确
        Sfc sfc1 = sfcMapper.selectById(HandleEnum.SFC.getHandle(site, sfc));
        if(sfc1 == null){
            throw new BaseException("sfc传递有误");
        }else {
            if(!HandleEnum.SHOP_ORDER.getHandle(site,shopOrder).equals(sfc1.getShopOrderBo())){
                throw new BaseException("工单传递有误");
            }

        }
        if(Constants.ABNORMAL_RESRCE.equals(category)){
            this.anomalyReportDevice(abnormalBill,abnormalBillDispose);
        }else if(Constants.ABNORMAL_OTHER.equals(category)){
            this.anomalyReportOther(abnormalBill,abnormalBillDispose,null,null);
        }

    }

    /**
     * 异常消息的发送
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @Override
    public boolean anomalyCreatedAndSendMessage(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        String locale = LocaleContextHolder.getLocale().getLanguage();
        LocalDateTime now = LocalDateTime.now();
        Message abnormalMessage = new Message();
        //发送消息的主键
        String handle = UUID.randomUUID().toString();
        abnormalMessage.setHandle(handle);
        //站点
        abnormalMessage.setSite(site);
        //异常单
        abnormalMessage.setObjectBo(abnormalBill.getHandle());
        //异常类型
        abnormalMessage.setType(abnormalBill.getType());
        //根据异常的类型找到消息的类型
        switch (abnormalMessage.getType()){
            case "Z"://质量异常
                abnormalMessage.setMessageType(Constants.ABNORMAL_MESSAGE_QUALITY);
                break;
            case "Q"://其他异常
                abnormalMessage.setMessageType(Constants.ABNORMAL_MESSAGE_OTHER);
                break;
            case "S"://设备异常
                abnormalMessage.setMessageType(Constants.ABNORMAL_MESSAGE_RESOURCE);
                break;
        }
        //设置发送的用户组
        switch (abnormalBill.getStatus()){
            case Constants.NEW://新建
                abnormalMessage.setSendUserGroup(abnormalBill.getReportSendUserGroup());
                abnormalMessage.setNode(Constants.ABNORMAL_MESSAGE_REPORT);
                break;
            case Constants.RESPONSE://响应中,设置响应超时的用户的发送的用户组为提报时要升级的用户组
                QueryWrapper<Message> wrapper = new QueryWrapper<>();
                Message abnormalMessage1 = new Message();
                abnormalMessage1.setSite(site);
                abnormalMessage1.setObjectBo(abnormalBill.getHandle());
                abnormalMessage1.setNode(Constants.ABNORMAL_MESSAGE_REPORT);
                wrapper.setEntity(abnormalMessage1);
                Message _abnormalMessage = messageMapper.selectOne(wrapper);
                abnormalMessage.setUpUserGroup(_abnormalMessage.getUpUserGroup());
                abnormalMessage.setUpMessageType(_abnormalMessage.getUpMessageType());
                abnormalMessage.setGrade(_abnormalMessage.getGrade());
                abnormalMessage.setUpDateTime(_abnormalMessage.getUpDateTime());
                abnormalMessage.setNode(Constants.ABNORMAL_MESSAGE_RESPONSE);
                break;
            case Constants.CORRECTION_CONFIRM://纠纷确认
                abnormalMessage.setSendUserGroup(abnormalBillDispose.getDutySendUserGroup());
                abnormalMessage.setNode(Constants.ABNORMAL_MESSAGE_CORRECTION_CONFIRM);
                break;
            case Constants.PROGRAM_CONFIRM://方案确认
                abnormalMessage.setSendUser(abnormalBillDispose.getResolveSendUser());
                abnormalMessage.setNode(Constants.ABNORMAL_MESSAGE_PROGRAM_CONFIRM);
                break;
            case Constants.CANCEL://取消
                abnormalMessage.setSendUserGroup(abnormalBill.getCancelSendUserGroup());
                abnormalMessage.setNode(Constants.ABNORMAL_MESSAGE_CANCEL);


                //异常取消的时候要关闭当前异常单下所有的异常信息
                UpdateWrapper<Message> messageTypeUpdateWrapper = new UpdateWrapper<>();
//              messageTypeUpdateWrapper.eq("SITE",site);
//              messageTypeUpdateWrapper.eq("ABNORMAL_BILL_BO",abnormalMessage.getAbnormalBillBo());
                Message abnormalMessage2 = new Message();
              //  abnormalMessage2.setAbnormalBillBo(abnormalMessage.getAbnormalBillBo());
                abnormalMessage2.setStatus("Y");
                messageTypeUpdateWrapper.eq("STATUS","N");
                messageTypeUpdateWrapper.eq("OBJECT_BO",abnormalMessage.getObjectBo());
                messageService.update(abnormalMessage2,messageTypeUpdateWrapper);
                break;
            case Constants.SHUT_DOWN:
                break;
        }

        String message = this.formatMessage(abnormalBill, abnormalMessage.getMessageType(), site);
        //得到消息触发的时间
        abnormalMessage.setResponseDateTime(now);
        //得到要发送的消息
        abnormalMessage.setContent(message);
        //得到升级的用户组
        //新建有、响应没有、纠防确认有、方案确认没有、取消没有、关闭没有
        if(Constants.NEW.equals(abnormalBill.getStatus()) || Constants.CORRECTION_CONFIRM.equals(abnormalBill.getStatus())){
            abnormalMessage.setUpUserGroup(abnormalMessage.getSendUserGroup()+"_SJ");
            //得到升级的时间和等级
            UserGroup customFieldByGroup = userGroupMapper.getCustomFieldByGroup(site, abnormalMessage.getSendUserGroup());
            if(customFieldByGroup != null && !StringUtil.isBlank(customFieldByGroup.getMessageNoticeTime())){
                abnormalMessage.setUpDateTime(abnormalMessage.getResponseDateTime().plusMinutes(Integer.parseInt(customFieldByGroup.getMessageNoticeTime())));
            }else{
                abnormalMessage.setStatus("Y");
            }
            if(customFieldByGroup != null && !StringUtil.isBlank(customFieldByGroup.getMessageNoticeLevel())){
                abnormalMessage.setGrade(Double.parseDouble(customFieldByGroup.getMessageNoticeLevel()));
            }
            //得到升级的消息类型
            abnormalMessage.setUpMessageType(abnormalMessage.getMessageType());
        }



        //得到异常节点

        //abnormalMessage.setAbnormalNode(Constants.ABNORMAL_MESSAGE_REPORT);

        //设置状态，新建是Y，响应为N，纠防确认是N，方案确认是Y，
        if(Constants.ABNORMAL_MESSAGE_RESPONSE.equals(abnormalMessage.getNode()) ||
                Constants.ABNORMAL_MESSAGE_CORRECTION_CONFIRM.equals(abnormalMessage.getNode())){
            abnormalMessage.setStatus("N");
        }else{
            abnormalMessage.setStatus("Y");
        }
        abnormalMessage.setCreatedUser(user);
        abnormalMessage.setCreatedDateTime(now);
        abnormalMessage.setModifiedUser(user);
        abnormalMessage.setModifiedDateTime(now);


        //判断，如果是方案确认，则发送用户，否则，除了节点是响应之外，则发送用户组
        if(Constants.ABNORMAL_MESSAGE_PROGRAM_CONFIRM.equals(abnormalMessage.getNode())){
            String sendUsers = abnormalMessage.getSendUser().replace(',', '|');
            activeMQUtil.wechatSendMessage(sendUsers,abnormalMessage.getContent());
        }else if(!Constants.ABNORMAL_MESSAGE_RESPONSE.equals(abnormalMessage.getNode())){
            List<NwaUser> nwaUsers = nwaUserService.checkUserGroup(site,abnormalMessage.getSendUserGroup());
            StringBuilder sendUsers = new StringBuilder();
            for (int i = 0; i < nwaUsers.size(); i++) {
                if(i == (nwaUsers.size() - 1)){
                    sendUsers.append(nwaUsers.get(i));
                }else{
                    sendUsers.append(nwaUsers.get(i)).append("|");
                }
            }
            activeMQUtil.wechatSendMessage(sendUsers.toString(),abnormalMessage.getContent());
        }
        messageService.saveOrUpdate(abnormalMessage);
        return true;
    }

    @Override
    public String formatMessage(AbnormalBill abnormalBill,String messageType, String site) {
        //找到消息内容
        String locale = LocaleContextHolder.getLocale().getLanguage();
        MessageType messageTypeEntity = new MessageType();
        messageTypeEntity.setSite(site);
        messageTypeEntity.setMessageType(messageType);
        QueryWrapper<MessageType> messageTypeQueryWrapper = new QueryWrapper<MessageType>();
        messageTypeQueryWrapper.setEntity(messageTypeEntity);
        MessageType sendMessageType = messageTypeMapper.selectOne(messageTypeQueryWrapper);
        if(sendMessageType == null){
            throw new BaseException("异常提报时找不到要发送的消息类型");
        }


        //查处所有要替换的消息内容
        AbnormalBillDisposeDto messageContent = abnormalBillDisposeMapper.findAllByAbnormalNo(abnormalBill, locale);
        //格式化消息
        String body = sendMessageType.getBody();
        //替换消息类型中的参数
        Map<String, String> messageMap = new HashMap<>();
        //根据车间找到对应车间的描述
//        WorkCenter workCenter = workCenterMapper.findWorkCenterDescriptionByWorkCenter(site, abnormalBill.getWorkCenter(), locale);
//        //根据资源找到对应的产线的描述
//        WorkCenter line = null;
//        if(StringUtil.isBlank(abnormalBill.getResrce())){
//            line = workCenterMapper.findWorkCenterDescriptionByResrce(HandleEnum.RESOURCE.getHandle(site, abnormalBill.getResrce()), locale);
//        }
//
//        //项目号
//        ShopOrderDto projectNo = shopOrderMapper.findByShopOrderBo(HandleEnum.SHOP_ORDER.getHandle(site, abnormalBill.getShopOrder()));
//        //物料
//        Item _item = itemMapper.selectById(locale, abnormalBill.getItemBo());
//        String item = "";
//        if(_item != null){
//            item = _item.getItem() + "/" + _item.getDescription();
//        }
//        if(workCenter != null && !StringUtil.isBlank(workCenter.getDescription())){
//            messageMap.put("WORKCENTER",workCenter.getDescription());
//        }
//        if(line != null && !StringUtil.isBlank(line.getDescription())){
//            messageMap.put("WORKCENTER_LIN",line.getDescription());
//        }
//        if(projectNo != null && !StringUtil.isBlank(projectNo.getItemNumber())){
//            messageMap.put("PROJECT",projectNo.getItemNumber());
//        }
//        if(!StringUtil.isBlank(abnormalBill.getWorkCenter())){
//            messageMap.put("SHOPORDER",abnormalBill.getWorkCenter());
//        }
//        messageMap.put("ITEM",item);
        //格式化之后的消息
        return StringUtils.format(body, messageMap);
    }

    @Override
    public List<AbnormalBill> getQCAbnormal() {
        return abnormalBillMapper.getQCAbnormalList(CommonMethods.getSite());
    }


    @Override
    public boolean jgProductionIsScrap(String site,String sfc) {
        //查询该sfc在质量异常异常方案是否是报废,如果是，则大于1
        Integer num = abnormalBillMapper.findAbnormalMethodBySFC(site, sfc);
        return num <= 0;
    }

    @Override
    public List<ScrapDto> getScrapDtoBysfc(String site, String sfc) {
        //判断该产品条码是否已经报废
//        if(sfcScrapMapper.selectById(HandleEnum.SFC_SCRAP.getHandle(site,sfc)) != null){
//            throw new BaseException("该产品条码已经报废");
//        }
        if(this.jgProductionIsScrap(site,sfc)){
            throw new BaseException("该产品条码的异常方案不是报废");
        }
        return abnormalBillMapper.findScrapDtoBySfc(site, sfc);
    }




}