package com.foreverwin.mesnac.anomaly.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.anomaly.dto.AbnormalBillDisposeDto;
import com.foreverwin.mesnac.anomaly.dto.AbnormalPlanDto;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalPlanMapper;
import com.foreverwin.mesnac.anomaly.model.AbnormalBill;
import com.foreverwin.mesnac.anomaly.model.AbnormalBillDispose;
import com.foreverwin.mesnac.anomaly.model.AbnormalPlan;
import com.foreverwin.mesnac.anomaly.service.AbnormalPlanService;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.mapper.MessageMapper;
import com.foreverwin.mesnac.common.model.Message;
import com.foreverwin.mesnac.common.service.MessageService;
import com.foreverwin.mesnac.common.util.ActiveMQUtil;
import com.foreverwin.mesnac.meapi.model.NwaUser;
import com.foreverwin.mesnac.meapi.service.NwaUserService;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 计划异常待办事项 服务实现类
 * </p>
 *
 * @author zjw
 * @since 2021-08-07
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class AbnormalPlanServiceImpl extends ServiceImpl<AbnormalPlanMapper, AbnormalPlan> implements AbnormalPlanService {


    @Autowired
    private AbnormalPlanMapper abnormalPlanMapper;

    @Autowired
    private MessageMapper messageMapper;

    @Autowired
    private MessageService messageService;

    @Autowired
    private NwaUserService nwaUserService;

    @Autowired
    private ActiveMQUtil activeMQUtil;


    @Override
    public IPage<AbnormalPlan> selectPage(FrontPage<AbnormalPlan> frontPage, AbnormalPlan abnormalPlan) {
        QueryWrapper<AbnormalPlan> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalPlan);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<AbnormalPlan> selectList(AbnormalPlan abnormalPlan) {
        QueryWrapper<AbnormalPlan> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalPlan);
        return super.list(queryWrapper);
    }


    @Override
    public void saveAbnormalPlan(AbnormalBillDisposeDto abnormalBillDisposeDto, LocalDateTime now) {
        String site = CommonMethods.getSite();
        AbnormalPlan abnormalPlan = new AbnormalPlan();
        String user = CommonMethods.getUser();
        //设置主键
        abnormalPlan.setHandle(HandleEnum.ABNORMAL_PLAN.getHandle(site,abnormalBillDisposeDto.getAbnormalNo()));
        abnormalPlan.setSite(site);
        abnormalPlan.setAbnormalNo(abnormalBillDisposeDto.getAbnormalNo());
        abnormalPlan.setStatus(Constants.UNTREATED);//未处理
        abnormalPlan.setWorkCenter(abnormalBillDisposeDto.getWorkCenter());
        abnormalPlan.setAbnormalMethod(abnormalBillDisposeDto.getAbnormalMethod());
        abnormalPlan.setShopOrder(abnormalBillDisposeDto.getShopOrder());
        abnormalPlan.setResrce(abnormalBillDisposeDto.getResrce());
        abnormalPlan.setItemBo(abnormalBillDisposeDto.getItemBo());
        abnormalPlan.setQty(Long.parseLong(abnormalBillDisposeDto.getNcQty()));
        //没有处理人和处理时间
//      abnormalPlan.setProcessor();
        abnormalPlan.setClosedUser(user);
        abnormalPlan.setClosedDateTime(now);
        abnormalPlan.setCreatedUser(user);
        abnormalPlan.setCreatedDateTime(now);
        abnormalPlan.setModifiedUser(user);
        abnormalPlan.setModifiedDateTime(now);
        this.saveOrUpdate(abnormalPlan);
    }

    @Override
    public void saveAbnormalPlan(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose, LocalDateTime now) {
        AbnormalBillDisposeDto abnormalBillDisposeDto = new AbnormalBillDisposeDto();
        abnormalBillDisposeDto.setHandle(abnormalBill.getHandle());
        abnormalBillDisposeDto.setAbnormalNo(abnormalBill.getAbnormalNo());
        abnormalBillDisposeDto.setStatus(abnormalBill.getStatus());
        abnormalBillDisposeDto.setWorkCenter(abnormalBill.getWorkCenter());
        abnormalBillDisposeDto.setAbnormalMethod(abnormalBillDispose.getAbnormalMethod());
        abnormalBillDisposeDto.setShopOrder(abnormalBill.getShopOrder());
        abnormalBillDisposeDto.setResrce(abnormalBill.getResrce());
        abnormalBillDisposeDto.setItem(abnormalBill.getItemBo());
        abnormalBillDisposeDto.setNcQty(abnormalBill.getNcQty());
        this.saveAbnormalPlan(abnormalBillDisposeDto,now);
    }

    @Override
    public List<AbnormalPlanDto> findAbnormalPlan(AbnormalPlanDto abnormalPlanDto) {
        String site = CommonMethods.getSite();
        String locale = LocaleContextHolder.getLocale().getLanguage();
        abnormalPlanDto.setSite(site);
        return abnormalPlanMapper.findAbnormalPlan(locale,abnormalPlanDto);
    }

    @Override
    public void processes(List<AbnormalPlan> abnormalPlanList) {
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        String user = CommonMethods.getUser();
        for (AbnormalPlan abnormalPlan : abnormalPlanList){
            Message message = new Message();
            String uuid = UUID.randomUUID().toString();
            message.setHandle(uuid);
            message.setSite(site);
            message.setObjectBo(HandleEnum.ABNORMAL_PLAN.getHandle(site,abnormalPlan.getAbnormalNo()));
            message.setMessageType(Constants.ABNORMAL_MESSAGE_PLAN);

            //设置消息格式content，没做


            message.setSendUserGroup(abnormalPlan.getSendUserGroup());
            message.setGrade(Double.parseDouble("1.0"));
            message.setResponseDateTime(now);
            message.setNode("PLAN");
            //设置为Y，则不进入计时器
            message.setStatus("Y");




            message.setCreatedUser(user);
            message.setCreatedDateTime(now);
            message.setModifiedUser(user);
            message.setModifiedDateTime(now);

            //发送消息
            List<NwaUser> nwaUsers = nwaUserService.checkUserGroup(site,message.getSendUserGroup());
            StringBuilder sendUsers = new StringBuilder();
            for (int i = 0; i < nwaUsers.size(); i++) {
                if(i == (nwaUsers.size() - 1)){
                    sendUsers.append(nwaUsers.get(i));
                }else{
                    sendUsers.append(nwaUsers.get(i)).append("|");
                }
            }
            activeMQUtil.wechatSendMessage(sendUsers.toString(),message.getContent());
            //保存
            messageService.saveOrUpdate(message);
        }
    }

}