package com.foreverwin.mesnac.common.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.common.model.InspectionItemAddition;
import com.foreverwin.mesnac.common.service.InspectionItemAdditionService;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 * @author Philip
 * @since 2021-06-09
 */
@RestController
@RequestMapping("/Z-INSPECTION-ITEM-ADDITION")
public class InspectionItemAdditionController {

    @Autowired
    public InspectionItemAdditionService inspectionItemAdditionService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getInspectionItemAdditionById(@PathVariable String id) {
        return R.ok( inspectionItemAdditionService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getInspectionItemAdditionList(InspectionItemAddition inspectionItemAddition){
        List<InspectionItemAddition> result;
        QueryWrapper<InspectionItemAddition> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(inspectionItemAddition);
        result = inspectionItemAdditionService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<InspectionItemAddition> frontPage, InspectionItemAddition inspectionItemAddition){
        IPage result;
        QueryWrapper<InspectionItemAddition> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(inspectionItemAddition);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(InspectionItemAddition::getHandle, frontPage.getGlobalQuery())
                .or().like(InspectionItemAddition::getInspectionItemBo, frontPage.getGlobalQuery())
                .or().like(InspectionItemAddition::getAdditionalObjectType, frontPage.getGlobalQuery())
                .or().like(InspectionItemAddition::getAdditionalObject, frontPage.getGlobalQuery())
                .or().like(InspectionItemAddition::getDescription, frontPage.getGlobalQuery())
                .or().like(InspectionItemAddition::getCreateUser, frontPage.getGlobalQuery())
                .or().like(InspectionItemAddition::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = inspectionItemAdditionService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param inspectionItemAddition  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody InspectionItemAddition inspectionItemAddition) {
        return R.ok(inspectionItemAdditionService.save(inspectionItemAddition));
    }

    /**
     * 修改
     * @param inspectionItemAddition  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody InspectionItemAddition inspectionItemAddition) {
        return R.ok(inspectionItemAdditionService.updateById(inspectionItemAddition));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(inspectionItemAdditionService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(inspectionItemAdditionService.removeByIds(ids));
    }
}