package com.foreverwin.mesnac.common.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.common.model.ProdReadyTaskDetail;
import com.foreverwin.mesnac.common.service.ProdReadyTaskDetailService;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 * @author Philip
 * @since 2021-06-23
 */
@RestController
@RequestMapping("/Z-PROD-READY-TASK-DETAIL")
public class ProdReadyTaskDetailController {

    @Autowired
    public ProdReadyTaskDetailService prodReadyTaskDetailService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getProdReadyTaskDetailById(@PathVariable String id) {
        return R.ok( prodReadyTaskDetailService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getProdReadyTaskDetailList(ProdReadyTaskDetail prodReadyTaskDetail){
        List<ProdReadyTaskDetail> result;
        QueryWrapper<ProdReadyTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(prodReadyTaskDetail);
        result = prodReadyTaskDetailService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ProdReadyTaskDetail> frontPage, ProdReadyTaskDetail prodReadyTaskDetail){
        IPage result;
        QueryWrapper<ProdReadyTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(prodReadyTaskDetail);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ProdReadyTaskDetail::getHandle, frontPage.getGlobalQuery())
                .or().like(ProdReadyTaskDetail::getProdReadyTaskBo, frontPage.getGlobalQuery())
                .or().like(ProdReadyTaskDetail::getInspectionItem, frontPage.getGlobalQuery())
                .or().like(ProdReadyTaskDetail::getResult, frontPage.getGlobalQuery())
                .or().like(ProdReadyTaskDetail::getRemarks, frontPage.getGlobalQuery())
                .or().like(ProdReadyTaskDetail::getCreateUser, frontPage.getGlobalQuery())
                .or().like(ProdReadyTaskDetail::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = prodReadyTaskDetailService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param prodReadyTaskDetail  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ProdReadyTaskDetail prodReadyTaskDetail) {
        return R.ok(prodReadyTaskDetailService.save(prodReadyTaskDetail));
    }

    /**
     * 修改
     * @param prodReadyTaskDetail  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ProdReadyTaskDetail prodReadyTaskDetail) {
        return R.ok(prodReadyTaskDetailService.updateById(prodReadyTaskDetail));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(prodReadyTaskDetailService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(prodReadyTaskDetailService.removeByIds(ids));
    }
}