package com.foreverwin.mesnac.common.service.impl;


import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONString;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.dto.LabelPrintDto;
import com.foreverwin.mesnac.common.model.PrintLog;
import com.foreverwin.mesnac.common.mapper.PrintLogMapper;
import com.foreverwin.mesnac.common.service.PrintLogService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.CommonMethods;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author Philip
 * @since 2021-07-16
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class PrintLogServiceImpl extends ServiceImpl<PrintLogMapper, PrintLog> implements PrintLogService {

    @Value("${print.server}")
    private String printServer;

    @Autowired
    private PrintLogMapper printLogMapper;

    @Autowired
    private PrintLogService printLogService;

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void savePrintLog(String site, String user, PrintLog printLog) {
        LocalDateTime dateTime = LocalDateTime.now();

        printLog.setSite(site);
        printLog.setCreateUser(user);
        printLog.setCreatedDateTime(dateTime);
        printLog.setModifyUser(user);
        printLog.setModifiedDateTime(dateTime);
        this.save(printLog);
    }

    @Override
    public void labelPrint(String site, String user, List<PrintLog> printList) {
        // 获取printParam，并拼接为json格式字符串
        StringBuffer stringBuffer = new StringBuffer("[");
        for (PrintLog printLog : printList) {
            String printParam = printLog.getPrintParam();
            stringBuffer.append(printParam.replace("[","").replace("]","")).append(",");
        }
        StringBuffer append = stringBuffer.deleteCharAt(stringBuffer.length() - 1).append("]");
        List<LabelPrintDto> list= JSONArray.parseArray(append.toString(), LabelPrintDto.class);
        // 打印机、打印模板
        for (int i = 0; i < list.size(); i++) {
            list.get(i).setPrinter(printList.get(i).getPrintName());
            list.get(i).setPrintTemplate(printList.get(i).getPrintTemplate());
        }
        String post = HttpUtil.post(printServer, JSON.toJSONString(list), 60000);
        /**
         * {"msg":"打印成功","code":0}
         * {"msg":"打印失败","code":500}
         * 根据返回值，更新打印机信息-printName及打印是否成功信息isPrint
         */
        JSONObject jsonObject = JSONObject.parseObject(post);
        Integer code = jsonObject.getInteger("code");
        String msg = jsonObject.getString("msg");
        if (code == 500){
            throw BusinessException.build(msg);
        }
        // 更新打印记录：是否已打印、使用的打印机、打印次数
        List<PrintLog> printLogList = new ArrayList<>();
        for (PrintLog printLog : printList) {
            PrintLog printLogOne = new PrintLog();
            printLogOne.setHandle(printLog.getHandle());
            printLogOne.setIsPrint("true");
            printLogOne.setPrintNum(printLog.getPrintNum()+1);
            if (printLog.getCategory().equals(Constants.PRINT_TYPE_INV)){
                printLogOne.setPrintName(printLog.getPrintName());
            }
            printLogList.add(printLogOne);
        }
        printLogService.updateBatchById(printLogList);

    }
}