package com.foreverwin.mesnac.dataimport.handler;

import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.dataimport.client.CustomDataWSClient;
import com.foreverwin.mesnac.dataimport.client.WorkCenterWSClient;
import com.foreverwin.mesnac.dataimport.handler.base.BaseHandler;
import com.foreverwin.mesnac.dataimport.reader.FileReader;
import com.foreverwin.mesnac.dataimport.reader.RowVisitor;
import com.foreverwin.mesnac.dataimport.reader.XlsxFileReaderMinth;
import com.foreverwin.mesnac.dataimport.service.MasterObjectDefine;
import com.foreverwin.mesnac.dataimport.util.AppUtil;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.I18nUtil;
import com.sap.me.plant.WorkCenterFullConfiguration;

import java.io.InputStream;
import java.util.Map;

public class WorkCenterHandler extends BaseHandler {

    @Override
    public String importFile(String site, InputStream inputStream, String fileType, String mode) {
        int row = 0;
        StringBuffer buffer = new StringBuffer();
        Integer[] failedNumber = new Integer[]{0};
        try {
            FileReader fileReader = new XlsxFileReaderMinth(fileType);
            fileReader.setMESLang(CustomDataWSClient.getSiteDefaultLang(site));
            RowVisitor<JSONObject> visitor = getRowVisitor(site, mode, buffer, failedNumber);
            row = fileReader.visitor(visitor).read(inputStream, getHeaders());
        } catch (Exception e) {
            buffer.append(e.getMessage() + "\n");
        }
        if (buffer.length() > 0) {
            buffer.insert(0, I18nUtil.getI18nText("MaterData.import.Summary", new Object[]{row + failedNumber[0], row, failedNumber[0]}) + "\n");
            throw BusinessException.build(buffer.toString());
        }
        buffer.insert(0, I18nUtil.getI18nText("MaterData.import.Summary", new Object[]{row + failedNumber[0], row, failedNumber[0]}) + "\n");
        return buffer.toString();
    }

    @Override
    public Map<String, String> getHeaders() {

        return MasterObjectDefine.getHeadsMapping("WORK_CENTER");
    }


    public RowVisitor<JSONObject> getRowVisitor(String site, String mode, StringBuffer buffer, Integer[] failedNumber) {

        return new RowVisitor<JSONObject>() {

            @Override
            public int visit(long index, JSONObject jsonObject) {
                Object[] params = new Object[10];
                params[0] = index;
                params[1] = jsonObject.getString("workCenter");

                boolean insert = false;
                try {
                    if (jsonObject.containsKey("workCenter") && jsonObject.getString("workCenter").trim().length() > 0) {
                        jsonObject.put("site", site);

                        String resultMessage = null;
                        WorkCenterFullConfiguration result = WorkCenterWSClient.find(jsonObject);
                        if (result != null && result.getWorkCenter() != null && result.getWorkCenter().length() > 0) {
                            if (AppUtil.canUpdate(mode)) {
                                resultMessage = WorkCenterWSClient.update(jsonObject, result);
                                if (resultMessage == null) {
                                    return 1;
                                }
                            }
                        } else {
                            if (AppUtil.canInsert(mode)) {
                                insert = true;
                                resultMessage = WorkCenterWSClient.insert(jsonObject);
                                if (resultMessage == null) {
                                    return 1;
                                }
                            } else if (AppUtil.canUpdate(mode)) {
                                resultMessage = "第" +index+ "行:" + I18nUtil.getI18nText("MaterData.import.upDateObjectNotExist") + "\n";
                            }
                        }

                        if (resultMessage != null) {
                            params[2] = resultMessage;
                            failedNumber[0]++;
                            if (insert) {
                                buffer.append("第" +index+ "行:" + I18nUtil.getI18nText("MaterData.import.insertfail.workcenter", params) + "\n");
                            } else {
                                buffer.append("第" +index+ "行:" + I18nUtil.getI18nText("MaterData.import.updatefail.workcenter", params) + "\n");
                            }
                        }
                    }
                    return 0;

                } catch (Exception e) {
                    params[2] = e.getMessage();
                    failedNumber[0]++;
                    if (insert) {
                        buffer.append("第" +index+ "行:" + I18nUtil.getI18nText("MaterData.import.insertfail.workcenter", new Object[]{e.getMessage()}) + "\n");
                    } else {
                        buffer.append("第" +index+ "行:" + I18nUtil.getI18nText("MaterData.import.updatefail.workcenter", new Object[]{e.getMessage()}) + "\n");
                    }
                }
                return 0;
            }

            @Override
            public String[] getHeader() {
                return null;
            }
        };
    }
}
