package com.foreverwin.mesnac.dispatch.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.dispatch.dto.UserResourceDto;
import com.foreverwin.mesnac.dispatch.mapper.UserResourceMapper;
import com.foreverwin.mesnac.dispatch.model.UserResource;
import com.foreverwin.mesnac.dispatch.service.UserResourceService;
import com.foreverwin.mesnac.meapi.model.NwaUser;
import com.foreverwin.mesnac.meapi.service.NwaUserService;
import com.foreverwin.mesnac.meapi.service.ResrceService;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author robert
 * @since 2021-06-24
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class UserResourceServiceImpl extends ServiceImpl<UserResourceMapper, UserResource> implements UserResourceService {


    @Autowired
    private UserResourceMapper userResourceMapper;


    @Autowired
    private ResrceService resrceService;

    @Autowired
    private NwaUserService nwaUserService;

    @Override
    public List<NwaUser> findEmployeeList(String site) {
        return userResourceMapper.findEmployeeList(site);
    }

    @Override
    public List<UserResource> findEnableList(String site, String resource, String user) {
        return userResourceMapper.findEnableList(site, resource, user);
    }

    @Override
    public HashMap<String,Object> findAll(UserResourceDto userResourceDto) {
        String site = CommonMethods.getSite();
        String resource = userResourceDto.getResrce();
        String userId = userResourceDto.getUserId();

        HashMap<String, Object> hashMap = new HashMap<>();;
        List<NwaUser> nwaUserList = nwaUserService.selectList(null);
        List<UserResourceDto> userResourceList = userResourceMapper.findAll(site, resource, userId);
        hashMap.put("users",nwaUserList);
        hashMap.put("userResources",userResourceList);
        return hashMap;

    }

    @Override
    public void saveAll(List<UserResourceDto> userResourceDtoList,String resource,String user) {
        List<UserResource> userResourceList = new ArrayList<>();
        String site = CommonMethods.getSite();
        userResourceMapper.deleteByUserAndResource(user,resource);
        for (int i = 0; i < userResourceDtoList.size(); i++) {
            UserResourceDto userResourceDto = userResourceDtoList.get(i);
            UserResource userResource = new UserResource();
            userResource.setSite(site);
            userResource.setResourceBo(HandleEnum.RESOURCE.getHandle(site, userResourceDto.getResrce()));
            userResource.setResrce(userResourceDto.getResrce());
            userResource.setResrceDescription(userResourceDto.getResrceDescription());
            if(resrceService.findByResrce(userResource.getResrce()) == null) {
                throw BusinessException.build("第"+(i+1)+"行的资源不存在");
            }
            userResource.setUserId(userResourceDto.getUserId());
            userResource.setTemporaryUser(userResourceDto.getTemporaryUser());
            userResource.setUserDescription(userResourceDto.getUserDescription());
            NwaUser userByUserName = nwaUserService.findUserByUserName(userResource.getUserId());
            if("false".equals(userResource.getTemporaryUser()) &&
                    userByUserName == null){
                throw BusinessException.build("第"+(i+1)+"行的用户不存在，必须为临时用户");
            }else if(userByUserName != null){
                //userByUserName.setFullName(userResource.getUserDescription());
                userResource.setUserDescription(userByUserName.getFullName());
                nwaUserService.saveOrUpdate(userByUserName);
            }
            userResource.setStatus(userResourceDto.getStatus());
            userResource.setHandle("UserResourceBo:"+userResource.getSite()+","+
                    userResource.getResrce()+","+userResource.getUserId());
            if(userResourceMapper.selectById(userResource.getHandle()) == null){
                userResource.setCreatedUser(CommonMethods.getUser());
                userResource.setCreatedDateTime(LocalDateTime.now());
            }
            userResource.setModifiedUser(CommonMethods.getUser());
            userResource.setModifiedDateTime(LocalDateTime.now());

            userResourceList.add(userResource);
        }
        if(userResourceList.size() > 0){
            this.saveOrUpdateBatch(userResourceList);
        }
    }

}