package com.foreverwin.mesnac.equip.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.equip.service.ResourceFaultReceiveService;
import com.foreverwin.mesnac.equip.model.ResourceFaultReceive;
import java.util.List;

/**
 *
 * @author pavel.Liu
 * @since 2021-07-13
 */
@RestController
@RequestMapping("/Z-RESOURCE-FAULT-RECEIVE")
public class ResourceFaultReceiveController {

    @Autowired
    public ResourceFaultReceiveService resourceFaultReceiveService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getResourceFaultReceiveById(@PathVariable String id) {
        return R.ok( resourceFaultReceiveService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getResourceFaultReceiveList(ResourceFaultReceive resourceFaultReceive){
        List<ResourceFaultReceive> result;
        QueryWrapper<ResourceFaultReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceFaultReceive);
        result = resourceFaultReceiveService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ResourceFaultReceive> frontPage, ResourceFaultReceive resourceFaultReceive){
        IPage result;
        QueryWrapper<ResourceFaultReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceFaultReceive);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ResourceFaultReceive::getHandle, frontPage.getGlobalQuery())
                .or().like(ResourceFaultReceive::getSite, frontPage.getGlobalQuery())
                .or().like(ResourceFaultReceive::getResrce, frontPage.getGlobalQuery())
                .or().like(ResourceFaultReceive::getFaultCode, frontPage.getGlobalQuery())
                .or().like(ResourceFaultReceive::getCreateUser, frontPage.getGlobalQuery())
                .or().like(ResourceFaultReceive::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = resourceFaultReceiveService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param resourceFaultReceive  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ResourceFaultReceive resourceFaultReceive) {
        return R.ok(resourceFaultReceiveService.save(resourceFaultReceive));
    }

    /**
     * 修改
     * @param resourceFaultReceive  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ResourceFaultReceive resourceFaultReceive) {
        return R.ok(resourceFaultReceiveService.updateById(resourceFaultReceive));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(resourceFaultReceiveService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(resourceFaultReceiveService.removeByIds(ids));
    }
}