package com.foreverwin.mesnac.equip.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.equip.service.ResourceInspectTaskSpareService;
import com.foreverwin.mesnac.equip.model.ResourceInspectTaskSpare;
import java.util.List;

/**
 *
 * @author pavel.Liu
 * @since 2021-07-01
 */
@RestController
@RequestMapping("/Z-RESOURCE-INSPECT-TASK-SPARE")
public class ResourceInspectTaskSpareController {

    @Autowired
    public ResourceInspectTaskSpareService resourceInspectTaskSpareService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getResourceInspectTaskSpareById(@PathVariable String id) {
        return R.ok( resourceInspectTaskSpareService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getResourceInspectTaskSpareList(ResourceInspectTaskSpare resourceInspectTaskSpare){
        List<ResourceInspectTaskSpare> result;
        QueryWrapper<ResourceInspectTaskSpare> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceInspectTaskSpare);
        result = resourceInspectTaskSpareService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ResourceInspectTaskSpare> frontPage, ResourceInspectTaskSpare resourceInspectTaskSpare){
        IPage result;
        QueryWrapper<ResourceInspectTaskSpare> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceInspectTaskSpare);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ResourceInspectTaskSpare::getHandle, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getSpareNo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getResourceInspectTaskBo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getItemBo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getDescription, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getUnit, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getRemark, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getCreateUser, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTaskSpare::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = resourceInspectTaskSpareService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param resourceInspectTaskSpare  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ResourceInspectTaskSpare resourceInspectTaskSpare) {
        return R.ok(resourceInspectTaskSpareService.save(resourceInspectTaskSpare));
    }

    /**
     * 修改
     * @param resourceInspectTaskSpare  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ResourceInspectTaskSpare resourceInspectTaskSpare) {
        return R.ok(resourceInspectTaskSpareService.updateById(resourceInspectTaskSpare));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(resourceInspectTaskSpareService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(resourceInspectTaskSpareService.removeByIds(ids));
    }
}