package com.foreverwin.mesnac.equip.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.equip.service.ResourceStatusReceiveService;
import com.foreverwin.mesnac.equip.model.ResourceStatusReceive;
import java.util.List;

/**
 *
 * @author pavel.Liu
 * @since 2021-07-13
 */
@RestController
@RequestMapping("/Z-RESOURCE-STATUS-RECEIVE")
public class ResourceStatusReceiveController {

    @Autowired
    public ResourceStatusReceiveService resourceStatusReceiveService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getResourceStatusReceiveById(@PathVariable String id) {
        return R.ok( resourceStatusReceiveService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getResourceStatusReceiveList(ResourceStatusReceive resourceStatusReceive){
        List<ResourceStatusReceive> result;
        QueryWrapper<ResourceStatusReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceStatusReceive);
        result = resourceStatusReceiveService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ResourceStatusReceive> frontPage, ResourceStatusReceive resourceStatusReceive){
        IPage result;
        QueryWrapper<ResourceStatusReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceStatusReceive);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ResourceStatusReceive::getHandle, frontPage.getGlobalQuery())
                .or().like(ResourceStatusReceive::getSite, frontPage.getGlobalQuery())
                .or().like(ResourceStatusReceive::getResrce, frontPage.getGlobalQuery())
                .or().like(ResourceStatusReceive::getStatus, frontPage.getGlobalQuery())
                .or().like(ResourceStatusReceive::getChangeStatus, frontPage.getGlobalQuery())
                .or().like(ResourceStatusReceive::getCreateUser, frontPage.getGlobalQuery())
                .or().like(ResourceStatusReceive::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = resourceStatusReceiveService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param resourceStatusReceive  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ResourceStatusReceive resourceStatusReceive) {
        return R.ok(resourceStatusReceiveService.save(resourceStatusReceive));
    }

    /**
     * 修改
     * @param resourceStatusReceive  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ResourceStatusReceive resourceStatusReceive) {
        return R.ok(resourceStatusReceiveService.updateById(resourceStatusReceive));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(resourceStatusReceiveService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(resourceStatusReceiveService.removeByIds(ids));
    }
}