package com.foreverwin.mesnac.equip.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.model.Resrce;
import com.foreverwin.mesnac.meapi.service.ResrceService;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.equip.model.ResourceStatusReceive;
import com.foreverwin.mesnac.equip.mapper.ResourceStatusReceiveMapper;
import com.foreverwin.mesnac.equip.service.ResourceStatusReceiveService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 设备状态表-接口 服务实现类
 * </p>
 *
 * @author pavel.Liu
 * @since 2021-07-13
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ResourceStatusReceiveServiceImpl extends ServiceImpl<ResourceStatusReceiveMapper, ResourceStatusReceive> implements ResourceStatusReceiveService {


    @Autowired
    private ResourceStatusReceiveMapper resourceStatusReceiveMapper;
    @Autowired
    private ResrceService resrceService;

    @Override
    public IPage<ResourceStatusReceive> selectPage(FrontPage<ResourceStatusReceive> frontPage, ResourceStatusReceive resourceStatusReceive) {
        QueryWrapper<ResourceStatusReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceStatusReceive);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<ResourceStatusReceive> selectList(ResourceStatusReceive resourceStatusReceive) {
        QueryWrapper<ResourceStatusReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceStatusReceive);
        return super.list(queryWrapper);
    }

    /**
     * 设备状态：运行/RUN(301|已启用)；停机/DOW(302|已禁用);维修/REPAIR(4/预定停机);封存/SEAL(303|保留);空闲/FREE(2|备用)
     * @param text
     * @return
     */
    @Override
    public String updateResourceStatusByMq(String text) {
        JSONObject jsonObject = JSONObject.parseObject(text);
        // 传输数据的唯一标识HANDLE，站点、设备编号、状态
        String handle = jsonObject.getString("TRANID");
        String site = jsonObject.getString("SITE");
        String resource = jsonObject.getString("RESOURCE");
        String sendStatus = jsonObject.getString("STATUS");
        // 数据传输时间
        String sendTime = jsonObject.getString("SEND_TIME");
        DateTimeFormatter dataFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.parse(sendTime,dataFormatter);

        if (StringUtil.isBlank(handle)) {
            throw BusinessException.build("唯一标识不能为空！");
        }
        if (StringUtil.isBlank(site)) {
            throw BusinessException.build("站点不能为空！");
        }
        if (StringUtil.isBlank(resource)) {
            throw BusinessException.build("设备编号不能为空！");
        }
        if (StringUtil.isBlank(sendStatus)) {
            throw BusinessException.build("设备状态不能为空！");
        }

        String status = "301";
        //设备状态转换
        switch (sendStatus) {
            case "RUN" :
                status = "301"; break;
            case "DOW" :
                status = "302"; break;
            case "SEAL" :
                status = "303"; break;
            case "FREE" :
                status = "2"; break;
            case "REPAIR" :
                status = "4"; break;
        }

        // 获取最新数据
        ResourceStatusReceive resourceStatus = resourceStatusReceiveMapper.getOneByMaxCreateDateTime(site);
        LocalDateTime oldDate = resourceStatus.getDateTime();
        resourceStatus.setChangeStatus(status);
        resourceStatus.setChangeDateTime(dateTime);
        Duration between = Duration.between(oldDate, dateTime);
        Long intervalTime = between.toMinutes();
        resourceStatus.setIntervalTime(intervalTime.doubleValue());
        resourceStatus.setModifiedDateTime(LocalDateTime.now());
        resourceStatus.setModifyUser("SITE_ADMIN");
        resourceStatusReceiveMapper.updateById(resourceStatus);
        // 数据库录入新数据
        ResourceStatusReceive resourceStatusReceive = new ResourceStatusReceive();
        resourceStatusReceive.setHandle(UUID.randomUUID().toString());
        resourceStatusReceive.setSite(site);
        resourceStatusReceive.setResrce(resource);
        resourceStatusReceive.setStatus(status);
        resourceStatusReceive.setDateTime(dateTime);
        resourceStatusReceive.setCreateUser("SITE_ADMIN");
        resourceStatusReceive.setCreatedDateTime(LocalDateTime.now());
        resourceStatusReceiveMapper.insert(resourceStatusReceive);
        // 修改设备状态
        Resrce resrce = resrceService.getById(HandleEnum.RESOURCE.getHandle(site, resource));
        resrce.setStatusBo(HandleEnum.STATUS.getHandle(site,status));
        resrce.setModifiedDateTime(LocalDateTime.now());
        resrceService.updateById(resrce);
        return handle;
    }

}