package com.foreverwin.mesnac.meapi.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.meapi.service.CustomFieldsService;
import com.foreverwin.mesnac.meapi.model.CustomFields;
import java.util.List;

/**
 *
 * @author LZP
 * @since 2021-06-01
 */
@RestController
@RequestMapping("/CUSTOM-FIELDS")
public class CustomFieldsController {

    @Autowired
    public CustomFieldsService customFieldsService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getCustomFieldsById(@PathVariable String id) {
        return R.ok( customFieldsService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getCustomFieldsList(CustomFields customFields){
        List<CustomFields> result;
        QueryWrapper<CustomFields> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(customFields);
        result = customFieldsService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<CustomFields> frontPage, CustomFields customFields){
        IPage result;
        QueryWrapper<CustomFields> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(customFields);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(CustomFields::getHandle, frontPage.getGlobalQuery())
                .or().like(CustomFields::getAttribute, frontPage.getGlobalQuery())
                .or().like(CustomFields::getValue, frontPage.getGlobalQuery())
    );
        }
        result = customFieldsService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param customFields  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody CustomFields customFields) {
        return R.ok(customFieldsService.save(customFields));
    }

    /**
     * 修改
     * @param customFields  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody CustomFields customFields) {
        return R.ok(customFieldsService.updateById(customFields));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(customFieldsService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(customFieldsService.removeByIds(ids));
    }
}