package com.foreverwin.mesnac.meapi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.meapi.model.DataField;
import com.foreverwin.mesnac.meapi.service.DataFieldService;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 * @author robert
 * @since 2021-06-29
 */
@RestController
@RequestMapping("/DATA-FIELD")
public class DataFieldController {

    @Autowired
    public DataFieldService dataFieldService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getDataFieldById(@PathVariable String id) {
        return R.ok( dataFieldService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getDataFieldList(DataField dataField){
        List<DataField> result;
        QueryWrapper<DataField> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(dataField);
        result = dataFieldService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<DataField> frontPage, DataField dataField){
        IPage result;
        QueryWrapper<DataField> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(dataField);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(DataField::getHandle, frontPage.getGlobalQuery())
                .or().like(DataField::getDataTypeBo, frontPage.getGlobalQuery())
                .or().like(DataField::getDataField, frontPage.getGlobalQuery())
                .or().like(DataField::getFieldType, frontPage.getGlobalQuery())
                .or().like(DataField::getValidationType, frontPage.getGlobalQuery())
                .or().like(DataField::getDataDefault, frontPage.getGlobalQuery())
                .or().like(DataField::getSite, frontPage.getGlobalQuery())
                .or().like(DataField::getBrowseActivityBo, frontPage.getGlobalQuery())
                .or().like(DataField::getIsBrowse, frontPage.getGlobalQuery())
                .or().like(DataField::getIsErp, frontPage.getGlobalQuery())
                .or().like(DataField::getMaskGroupBo, frontPage.getGlobalQuery())
                .or().like(DataField::getDescription, frontPage.getGlobalQuery())
                .or().like(DataField::getValidationActivityBo, frontPage.getGlobalQuery())
                .or().like(DataField::getCategory, frontPage.getGlobalQuery())
    );
        }
        result = dataFieldService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param dataField  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody DataField dataField) {
        return R.ok(dataFieldService.save(dataField));
    }

    /**
     * 修改
     * @param dataField  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody DataField dataField) {
        return R.ok(dataFieldService.updateById(dataField));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(dataFieldService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(dataFieldService.removeByIds(ids));
    }


}