package com.foreverwin.mesnac.meapi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.meapi.model.MessageType;
import com.foreverwin.mesnac.meapi.service.MessageTypeService;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 * @author Philip
 * @since 2021-07-20
 */
@RestController
@RequestMapping("/MESSAGE-TYPE")
public class MessageTypeController {

    @Autowired
    public MessageTypeService messageTypeService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getMessageTypeById(@PathVariable String id) {
        return R.ok( messageTypeService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getMessageTypeList(MessageType messageType){
        List<MessageType> result;
        QueryWrapper<MessageType> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(messageType);
        result = messageTypeService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<MessageType> frontPage, MessageType messageType){
        IPage result;
        QueryWrapper<MessageType> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(messageType);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(MessageType::getHandle, frontPage.getGlobalQuery())
                .or().like(MessageType::getSite, frontPage.getGlobalQuery())
                .or().like(MessageType::getMessageType, frontPage.getGlobalQuery())
                .or().like(MessageType::getDescription, frontPage.getGlobalQuery())
                .or().like(MessageType::getSeverity, frontPage.getGlobalQuery())
                .or().like(MessageType::getProcessWorkflowBo, frontPage.getGlobalQuery())
                .or().like(MessageType::getEnabled, frontPage.getGlobalQuery())
                .or().like(MessageType::getSubject, frontPage.getGlobalQuery())
                .or().like(MessageType::getBody, frontPage.getGlobalQuery())
                .or().like(MessageType::getAutoClose, frontPage.getGlobalQuery())
                .or().like(MessageType::getAutoCloseIntervalUnit, frontPage.getGlobalQuery())
                .or().like(MessageType::getAutoClosedByWf, frontPage.getGlobalQuery())
                .or().like(MessageType::getSignalFlag, frontPage.getGlobalQuery())
    );
        }
        result = messageTypeService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param messageType  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody MessageType messageType) {
        return R.ok(messageTypeService.save(messageType));
    }

    /**
     * 修改
     * @param messageType  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody MessageType messageType) {
        return R.ok(messageTypeService.updateById(messageType));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(messageTypeService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(messageTypeService.removeByIds(ids));
    }
}