package com.foreverwin.mesnac.meapi.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.meapi.service.ResourceTypeResourceService;
import com.foreverwin.mesnac.meapi.model.ResourceTypeResource;
import java.util.List;

/**
 *
 * @author pavel.Liu
 * @since 2021-07-09
 */
@RestController
@RequestMapping("/RESOURCE-TYPE-RESOURCE")
public class ResourceTypeResourceController {

    @Autowired
    public ResourceTypeResourceService resourceTypeResourceService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getResourceTypeResourceById(@PathVariable String id) {
        return R.ok( resourceTypeResourceService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getResourceTypeResourceList(ResourceTypeResource resourceTypeResource){
        List<ResourceTypeResource> result;
        QueryWrapper<ResourceTypeResource> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceTypeResource);
        result = resourceTypeResourceService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ResourceTypeResource> frontPage, ResourceTypeResource resourceTypeResource){
        IPage result;
        QueryWrapper<ResourceTypeResource> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceTypeResource);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ResourceTypeResource::getHandle, frontPage.getGlobalQuery())
                .or().like(ResourceTypeResource::getResourceTypeBo, frontPage.getGlobalQuery())
                .or().like(ResourceTypeResource::getResourceBo, frontPage.getGlobalQuery())
    );
        }
        result = resourceTypeResourceService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param resourceTypeResource  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ResourceTypeResource resourceTypeResource) {
        return R.ok(resourceTypeResourceService.save(resourceTypeResource));
    }

    /**
     * 修改
     * @param resourceTypeResource  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ResourceTypeResource resourceTypeResource) {
        return R.ok(resourceTypeResourceService.updateById(resourceTypeResource));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(resourceTypeResourceService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(resourceTypeResourceService.removeByIds(ids));
    }
}