package com.foreverwin.mesnac.meapi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.meapi.model.RouterSubstep;
import com.foreverwin.mesnac.meapi.service.RouterSubstepService;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 * @author Robert
 * @since 2021-06-03
 */
@RestController
@RequestMapping("/ROUTER-SUBSTEP")
public class RouterSubstepController {

    @Autowired
    public RouterSubstepService routerSubstepService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getRouterSubstepById(@PathVariable String id) {
        return R.ok( routerSubstepService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getRouterSubstepList(RouterSubstep routerSubstep){
        List<RouterSubstep> result;
        QueryWrapper<RouterSubstep> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(routerSubstep);
        result = routerSubstepService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<RouterSubstep> frontPage, RouterSubstep routerSubstep){
        IPage result;
        QueryWrapper<RouterSubstep> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(routerSubstep);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(RouterSubstep::getHandle, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getRouterStepBo, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getSubstep, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getSubstepId, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getSubstepGroup, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getSubstepType, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getDescription, frontPage.getGlobalQuery())
                .or().like(RouterSubstep::getLongDescription, frontPage.getGlobalQuery())
    );
        }
        result = routerSubstepService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param routerSubstep  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody RouterSubstep routerSubstep) {
        return R.ok(routerSubstepService.save(routerSubstep));
    }

    /**
     * 修改
     * @param routerSubstep  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody RouterSubstep routerSubstep) {
        return R.ok(routerSubstepService.updateById(routerSubstep));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(routerSubstepService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(routerSubstepService.removeByIds(ids));
    }
    @GetMapping("getRouterStepBo")
    public R getRouterStepBo(FrontPage<RouterSubstep> page,String item,String operation){
        String globalQuery = page.getGlobalQuery();
        if(StringUtils.notBlank(globalQuery)){
            globalQuery = globalQuery.toUpperCase();
        }
        return R.ok(routerSubstepService.getRouterStepBo(page.getPagePlus(),globalQuery,item,operation));
    }


}