package com.foreverwin.mesnac.meapi.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.meapi.service.WorkCenterMemberService;
import com.foreverwin.mesnac.meapi.model.WorkCenterMember;
import java.util.List;

/**
 *
 * @author Leon.L
 * @since 2021-05-26
 */
@RestController
@RequestMapping("/WORK-CENTER-MEMBER")
public class WorkCenterMemberController {

    @Autowired
    public WorkCenterMemberService workCenterMemberService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getWorkCenterMemberById(@PathVariable String id) {
        return R.ok( workCenterMemberService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getWorkCenterMemberList(WorkCenterMember workCenterMember){
        List<WorkCenterMember> result;
        QueryWrapper<WorkCenterMember> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workCenterMember);
        result = workCenterMemberService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<WorkCenterMember> frontPage, WorkCenterMember workCenterMember){
        IPage result;
        QueryWrapper<WorkCenterMember> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workCenterMember);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(WorkCenterMember::getHandle, frontPage.getGlobalQuery())
                .or().like(WorkCenterMember::getWorkCenterBo, frontPage.getGlobalQuery())
                .or().like(WorkCenterMember::getWorkCenterOrResourceGbo, frontPage.getGlobalQuery())
                .or().like(WorkCenterMember::getPrimaryWorkCenter, frontPage.getGlobalQuery())
                .or().like(WorkCenterMember::getDefaultMember, frontPage.getGlobalQuery())
    );
        }
        result = workCenterMemberService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param workCenterMember  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody WorkCenterMember workCenterMember) {
        return R.ok(workCenterMemberService.save(workCenterMember));
    }

    /**
     * 修改
     * @param workCenterMember  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody WorkCenterMember workCenterMember) {
        return R.ok(workCenterMemberService.updateById(workCenterMember));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(workCenterMemberService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(workCenterMemberService.removeByIds(ids));
    }
}