package com.foreverwin.mesnac.production.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.production.service.LoadInventoryLogService;
import com.foreverwin.mesnac.production.model.LoadInventoryLog;
import java.util.List;

/**
 *
 * @author Philip
 * @since 2021-07-09
 */
@RestController
@RequestMapping("/Z-LOAD-INVENTORY-LOG")
public class LoadInventoryLogController {

    @Autowired
    public LoadInventoryLogService loadInventoryLogService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getLoadInventoryLogById(@PathVariable String id) {
        return R.ok( loadInventoryLogService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getLoadInventoryLogList(LoadInventoryLog loadInventoryLog){
        List<LoadInventoryLog> result;
        QueryWrapper<LoadInventoryLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(loadInventoryLog);
        result = loadInventoryLogService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<LoadInventoryLog> frontPage, LoadInventoryLog loadInventoryLog){
        IPage result;
        QueryWrapper<LoadInventoryLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(loadInventoryLog);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(LoadInventoryLog::getHandle, frontPage.getGlobalQuery())
                .or().like(LoadInventoryLog::getSite, frontPage.getGlobalQuery())
                .or().like(LoadInventoryLog::getInventoryId, frontPage.getGlobalQuery())
                .or().like(LoadInventoryLog::getResrce, frontPage.getGlobalQuery())
                .or().like(LoadInventoryLog::getItem, frontPage.getGlobalQuery())
                .or().like(LoadInventoryLog::getActionCode, frontPage.getGlobalQuery())
                .or().like(LoadInventoryLog::getCreateUser, frontPage.getGlobalQuery())
    );
        }
        result = loadInventoryLogService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param loadInventoryLog  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody LoadInventoryLog loadInventoryLog) {
        return R.ok(loadInventoryLogService.save(loadInventoryLog));
    }

    /**
     * 修改
     * @param loadInventoryLog  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody LoadInventoryLog loadInventoryLog) {
        return R.ok(loadInventoryLogService.updateById(loadInventoryLog));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(loadInventoryLogService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(loadInventoryLogService.removeByIds(ids));
    }
}