package com.foreverwin.mesnac.production.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.constant.IntegrationTypeConstant;
import com.foreverwin.mesnac.common.enums.DispatchStatusEnum;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.erp.ZprodordconfStruIn;
import com.foreverwin.mesnac.common.erp.ZprodordconfStruOut;
import com.foreverwin.mesnac.common.erp.holders.TableOfZprodordconfStruInHolder;
import com.foreverwin.mesnac.common.erp.holders.TableOfZprodordconfStruOutHolder;
import com.foreverwin.mesnac.common.model.InspectionItemDetail;
import com.foreverwin.mesnac.common.model.InspectionTask;
import com.foreverwin.mesnac.common.service.CommonService;
import com.foreverwin.mesnac.common.service.InspectionItemService;
import com.foreverwin.mesnac.common.service.InspectionTaskService;
import com.foreverwin.mesnac.common.service.SfcDispatchCommonService;
import com.foreverwin.mesnac.common.util.ERPAPI;
import com.foreverwin.mesnac.common.util.ExceptionUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.integration.model.IntegrationLog;
import com.foreverwin.mesnac.integration.service.IntegrationLogService;
import com.foreverwin.mesnac.meapi.dto.BomComponentDto;
import com.foreverwin.mesnac.meapi.dto.SfcDto;
import com.foreverwin.mesnac.meapi.dto.WorkCenterDto;
import com.foreverwin.mesnac.meapi.model.*;
import com.foreverwin.mesnac.meapi.service.*;
import com.foreverwin.mesnac.production.mapper.SfcCrossMapper;
import com.foreverwin.mesnac.production.mapper.SfcDataAssembleMapper;
import com.foreverwin.mesnac.production.model.LoadInventory;
import com.foreverwin.mesnac.production.service.LoadInventoryService;
import com.foreverwin.mesnac.production.service.PodTemplateService;
import com.foreverwin.mesnac.production.service.SfcCrossService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.meext.MEServices;
import com.foreverwin.modular.core.util.CommonMethods;
import com.sap.me.production.AssembleComponentsRequest;
import com.sap.me.production.AssemblyComponent;
import com.sap.me.production.AssemblyDataField;
import com.sap.me.production.AssemblyServiceInterface;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;


@Service
@Transactional(rollbackFor = Exception.class)
public class PodTemplateServiceImpl implements PodTemplateService {
    @Autowired
    private ResrceService resrceService;
    @Autowired
    private SfcService sfcService;
    @Autowired
    private IntegrationLogService integrationLogService;
    @Autowired
    private RouterService routerService;
    @Autowired
    private ItemService itemService;
    @Autowired
    private SfcBomService sfcBomService;
    @Autowired
    private CommonService commonService;
    @Autowired
    private SfcCrossMapper sfcCrossMapper;
    @Autowired
    private SfcCrossService sfcCrossService;
    @Autowired
    private OperationService operationService;
    @Autowired
    private BomComponentService bomComponentService;
    @Autowired
    private SfcDataAssembleMapper sfcDataAssembleMapper;
    @Autowired
    private InspectionTaskService inspectionTaskService;
    @Autowired
    private InspectionItemService inspectionItemService;
    @Autowired
    private SfcDispatchCommonService sfcDispatchCommonService;
    @Autowired
    private LoadInventoryService loadInventoryService;

    @Override
    public Map<String, Object> resrceEnter(WorkCenterDto workCenterDto) {
        String site = CommonMethods.getSite();
        String resrce = workCenterDto.getResrce();
        String resrceBO = HandleEnum.RESOURCE.getHandle(site, resrce);
        Resrce model = resrceService.getById(resrceBO);
        if (model == null) {
            throw new BaseException("资源 " + resrce + " 不存在");
        }
        String workCenter = workCenterDto.getWorkCenter();
        //校验
        String workShopBo = commonService.getWorkShopBo(resrceBO);
        if (StringUtil.isBlank(workShopBo)) {
            throw new BaseException("资源 " + resrce + " 未匹配到车间");
        }
        if (!workCenter.equals(StringUtil.trimHandle(workShopBo))) {
            throw new BaseException("资源 " + resrce + " 与车间不匹配");
        }
        //查询在该设备存在活动中的SFC
        List<SfcDto> sfcList = sfcCrossService.getSfcListByResrce(site, resrce);
        Map<String, Object> resultMap = new HashMap<String, Object>();
        resultMap.put("SFC_LIST", sfcList);
        return resultMap;
    }

    @Override
    public Map<String, Object> sfcEnter(SfcDto sfcDto) {
        String site = CommonMethods.getSite();
        String sfc = sfcDto.getSfc();
        String locale = LocaleContextHolder.getLocale().getLanguage();
        String resrce = sfcDto.getResrce();
        //校验产品条码是否存在
        Sfc sfcById = sfcService.getById(HandleEnum.SFC.getHandle(site, sfc));
        if (sfcById == null) {
            throw new BaseException("产品条码" + sfc + "不存在");
        }
        String status = StringUtil.trimHandle(sfcById.getStatusBo());
        if (!status.equals("403") && !status.equals("401") && !status.equals("402")) {
            throw new BaseException("产品条码" + sfc + "不在工作中或排队中");
        }
        //校验产品条码是否在当前工序排队
        Operation operationBySfcBo = commonService.getOperationBySfcBo(sfcById.getHandle());
       /* if (StringUtil.notBlank(operation)&&!operationBySfcBo.getOperation().equals(operation)){
            throw new BaseException("产品条码"+sfc+"在"+operationBySfcBo.getOperation()+"/"+operationBySfcBo.getDescription()+"工序");
        }*/
        //校验SFC+工序+设备+STEP_ID是否与当前设备确认的派工单匹配
        String sfcDispatchStatus = sfcCrossMapper.getSfcDispatch(site, sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId(), resrce);
        if (StringUtil.isBlank(sfcDispatchStatus)) {
            throw new BaseException("产品条码与当前设备工序确认的派工单不匹配");
        }

        //构造前台所需要的数据
        Map<String, Object> resultMap = sfcCrossMapper.querySfcData(site, locale, sfcDto);
        if (resultMap == null) {
            throw new BaseException("根据当前资源未找到条码[" + sfc + "]的基本信息！");
        }
        //List<Map<String, Object>> substepList = sfcCrossMapper.querySfcStep(site, sfc, operationBySfcBo.getOperation());
        List<Map<String, Object>> substepList = new ArrayList<>();
        resultMap.put("SFC_STEP_LIST", substepList);

        //是否需要自检和互检
        String isCreateH = "N";
        String isCreateZ = "N";
        resultMap.put("IS_CREATE_W", "N");
        List<InspectionItemDetail> inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId(), Constants.INSPECTION_TYPE_H);
        if (inspectionItemDetails.size() > 0 && inspectionItemDetails.get(0) != null) {
            InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_H, sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId());
            if (createTask == null || !createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)) {
                isCreateH = "Y";
            }

        }
        inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId(), Constants.INSPECTION_TYPE_Z);
        if (inspectionItemDetails.size() > 0 && inspectionItemDetails.get(0) != null) {
            InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_Z, sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId());
            if (createTask == null || !createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)) {
                isCreateZ = "Y";
            }
        }
        resultMap.put("IS_CREATE_H", isCreateH);
        resultMap.put("IS_CREATE_Z", isCreateZ);
        return resultMap;
    }

    @Override
    public Object getSfcInfo(SfcDto sfcDto) {
        String site = CommonMethods.getSite();
        String operation = sfcDto.getOperation();
        String sfc = sfcDto.getSfc();
        Map<String, Object> substepList = sfcCrossMapper.getSfcInfo(site, sfc, operation);
        return substepList;
    }

    @Override
    public void sfcStart(Map<String, Object> map) {
        List<SfcDto> sfcDtoList = (List<SfcDto>) map.get("sfcDtoList");
        ObjectMapper mapper = new ObjectMapper();
        sfcDtoList = mapper.convertValue(sfcDtoList, new TypeReference<List<SfcDto>>() {
        });
        String resrce = (String) map.get("resrce");
        String site = CommonMethods.getSite();

        if (sfcDtoList == null || sfcDtoList.size() < 1) {
            throw new BaseException("作业列表不能为空");
        }
        if (StringUtil.isBlank(resrce)) {
            throw new BaseException("资源不能为空");
        }
        //是否设备点检
        sfcCrossService.resourceCheck(resrce);
        sfcDtoList.forEach(sfcDto -> {
            String operation = sfcDto.getOperation();
            Operation currentRevisionRef = operationService.getCurrentRevisionRef(site, operation);
            String sfc = sfcDto.getSfc();
            String stepId = sfcDto.getStepId();
            String dispatchNo = sfcDto.getDispatchNo();
            Sfc sfcServiceById = sfcService.getById(HandleEnum.SFC.getHandle(site, sfc));
            BigDecimal qty = new BigDecimal(sfcServiceById.getQty().toString());
            //是否有互检检验项目
            List<InspectionItemDetail> inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operation, stepId, Constants.INSPECTION_TYPE_H);
            if (inspectionItemDetails.size() > 0 && inspectionItemDetails.get(0) != null) {
                //是否有互检检验任务及已完成
                InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_H, sfc, operation, stepId);
                if (createTask == null || !createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)) {
                    throw new BaseException("请完成互检检验任务");
                }
                if (!createTask.getResult().equals(Constants.RSESULT_OK)) {
                    throw new BaseException("互检任务不合格，不能开始请检查");
                }
            }
            //是否完成专检
            QueryWrapper<InspectionTask> queryWrapper=new QueryWrapper<>();
            queryWrapper.eq(InspectionTask.CATEGORY,Constants.INSPECTION_TYPE_P);
            queryWrapper.eq(InspectionTask.SFC,sfc);
            queryWrapper.ne(InspectionTask.RESULT,Constants.RSESULT_OK);
            List<InspectionTask> list = inspectionTaskService.list(queryWrapper);
            if (!list.isEmpty()) {
                throw new BaseException("请完成专检检验任务");
            }
            try {
                sfcCrossService.startAction(site, currentRevisionRef.getHandle(), resrce, sfcServiceById.getHandle(), qty);
            } catch (Exception e) {
                ExceptionUtil.throwException(e);
            }
            //更改派工单状态
            sfcDispatchCommonService.updateSfcDispatchStatus(site, CommonMethods.getUser(), dispatchNo, DispatchStatusEnum.START.getCode());

        });
    }

    @Override
    public void sfcComplete(Map<String, Object> map) {
        List<SfcDto> sfcDtoList = (List<SfcDto>) map.get("sfcDtoList");
        ObjectMapper mapper = new ObjectMapper();
        sfcDtoList = mapper.convertValue(sfcDtoList, new TypeReference<List<SfcDto>>() {});
        List<Map<String,Object>> postMapList=new ArrayList<>();
        String resrce = (String) map.get("resrce");
        if (sfcDtoList == null || sfcDtoList.size() < 1) {
            throw new BaseException("作业列表不能为空");
        }
        if (StringUtil.isBlank(resrce)) {
            throw new BaseException("资源不能为空");
        }
        for (SfcDto sfcDto:sfcDtoList){
            Map<String,Object> postMap=new HashMap<>();
            String site = CommonMethods.getSite();
            String operation = sfcDto.getOperation();
            Operation currentRevisionRef = operationService.getCurrentRevisionRef(site, operation);
            String sfc = sfcDto.getSfc();
            String stepId = sfcDto.getStepId();

            String dispatchNo = sfcDto.getDispatchNo();
            Sfc sfcServiceById = sfcService.getById(HandleEnum.SFC.getHandle(site, sfc));
            BigDecimal qty = new BigDecimal(sfcServiceById.getQty().toString());
            //是否有自检检验项目
            List<InspectionItemDetail> inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operation, stepId, Constants.INSPECTION_TYPE_Z);
            if (inspectionItemDetails.size() > 0 && inspectionItemDetails.get(0) != null) {
                //是否有自检检验任务
                InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_Z, sfc, operation, stepId);
                if (createTask == null || !createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)) {
                    throw new BaseException("请完成自检检验任务");
                }
                if (!createTask.getResult().equals(Constants.RSESULT_OK)) {
                    throw new BaseException("自检任务不合格，不能完成请检查");
                }
            }
            //是否装配完成
            List<Map<String, Object>> assembleList = sfcDataAssembleMapper.querySfcAssemble(HandleEnum.SFC_DISPATCH.getHandle(site,dispatchNo));
            if (assembleList!=null&&assembleList.size()>0){
                for (Map<String,Object> objectMap:assembleList){
                    String  isAssemble= (String) objectMap.get("IS_ASSEMBLE");
                    if (!isAssemble.equals("GREEN")){
                        throw new BaseException("未装配完成");
                    }
                }
            }
            //物料消耗
            materialConsumption(site, operation, HandleEnum.SFC.getHandle(site, sfc), stepId, resrce);
            try {
                //专检创建
                Map<String, Object> paramMap=new HashMap<>();
                paramMap.put("SFC",sfc);
                paramMap.put("SHOP_ORDER",StringUtil.trimHandle(sfcServiceById.getShopOrderBo()));
                paramMap.put("OPERATION",operation);
                paramMap.put("STEP_ID",stepId);
                String workShopBo = commonService.getWorkShopBo(HandleEnum.RESOURCE.getHandle(site, resrce));
                paramMap.put("WORK_CENTER",StringUtil.trimHandle(workShopBo));
                paramMap.put("RESRCE",resrce);
                paramMap.put("SFC_DISPATCH_DETAIL_BO",HandleEnum.SFC_DISPATCH.getHandle(site,dispatchNo));

                inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc,operation, stepId, Constants.INSPECTION_TYPE_P);
                if (inspectionItemDetails.size() > 0 && inspectionItemDetails.get(0) != null) {
                    paramMap.put("CATEGORY",Constants.INSPECTION_TYPE_P);
                    inspectionTaskService.createTask(paramMap);
                }else {
                    paramMap.put("CATEGORY",Constants.INSPECTION_TYPE_P);
                    inspectionTaskService.createCompleteTask(paramMap);
                }
                //首件创建
                inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc,operation, stepId, Constants.INSPECTION_TYPE_S);
                if (inspectionItemDetails.size() > 0 && inspectionItemDetails.get(0) != null) {
                    paramMap.put("CATEGORY",Constants.INSPECTION_TYPE_S);
                    inspectionTaskService.createTask(paramMap);
                }
                //计算工时
                LocalDateTime startTime = sfcService.getSfcStartTime(HandleEnum.SFC.getHandle(site, sfc));
                long workHourSeconds = Duration.between(startTime, LocalDateTime.now()).getSeconds();
                BigDecimal workHour = new BigDecimal(workHourSeconds).divide(BigDecimal.valueOf(3600), 2, RoundingMode.HALF_UP);
                postMap.put("sfc",sfc);
                postMap.put("stepId",stepId);
                postMap.put("qty",qty);
                postMap.put("scrapQty",BigDecimal.ZERO);
                postMap.put("workHour",workHour);
                postMapList.add(postMap);
                sfcCrossService.completeAction(site, currentRevisionRef.getHandle(), resrce, sfcServiceById.getHandle(), qty);
                //更改派工单状态
                sfcDispatchCommonService.updateSfcDispatchStatus(site, CommonMethods.getUser(), dispatchNo, DispatchStatusEnum.COMPLETE.getCode());

            } catch (Exception e) {
                ExceptionUtil.throwException(e);
            }
        }
        //报工
        //sendErp(postMapList);
    }

    public void materialConsumption(String site, String operation, String sfcBo, String stepId, String resource) {
        Sfc sfcServiceById = sfcService.getById(sfcBo);
        SfcBom queryEntity = new SfcBom();
        queryEntity.setSfcBo(sfcBo);
        List<SfcBom> sfcBoms = sfcBomService.selectList(queryEntity);
        if (sfcBoms.isEmpty()) {
            return;
        }
        //查询bom组件
        List<BomComponentDto> bomComponents = bomComponentService.getSfcBomComponent(sfcBo);
        if (bomComponents.isEmpty()) {
            return;
        }
        for (BomComponentDto BomComponentDto : bomComponents) {
            String accessoryType = BomComponentDto.getAccessoryType();
            String componentGbo = BomComponentDto.getComponentGbo();
            Item currentReversionItem = itemService.selectCurrent(site, StringUtil.trimHandle(componentGbo));
            //消耗辅料类型为空或null
            if ((accessoryType == null || accessoryType.equals("0")) && stepId.equals(BomComponentDto.getStepId())) {
                //查询设备上的料
                QueryWrapper<LoadInventory> wrapper = new QueryWrapper<>();
                wrapper.eq(LoadInventory.RESRCE, resource);
                wrapper.eq(LoadInventory.ITEM, StringUtil.trimHandle(componentGbo));
                wrapper.orderByAsc(LoadInventory.CREATED_DATE_TIME);
                List<LoadInventory> inventoryList = loadInventoryService.list(wrapper);
                if (inventoryList.isEmpty()) {
                    throw new BaseException("设备没有上" + StringUtil.trimHandle(componentGbo) + "/" + currentReversionItem.getDescription() + "料");
                }
                BigDecimal needQty = new BigDecimal(BomComponentDto.getQty()).multiply(new BigDecimal(sfcServiceById.getQty()));
                for (LoadInventory loadInventory : inventoryList) {
                    BigDecimal qtyOnHand = loadInventory.getQtyOnHand();
                    BigDecimal costQty;
                    if (needQty.doubleValue() > qtyOnHand.doubleValue()) {
                        costQty = qtyOnHand;
                        needQty = needQty.subtract(costQty);
                    } else {
                        costQty = needQty;
                        needQty = BigDecimal.ZERO;
                    }
                    //删除上料数据
                    Inventory inventory = loadInventoryService.consumption(loadInventory, costQty);
                    //消耗
                    try {
                        assembly(site, BomComponentDto.getHandle(), operation, inventory, sfcBo, resource, costQty, stepId, sfcServiceById.getShopOrderBo());
                    } catch (Exception e) {
                        ExceptionUtil.throwException(e);
                    }
                    if (needQty.doubleValue() == 0) {
                        break;
                    }
                }
                if (needQty.doubleValue() > 0) {
                    throw new BaseException("组件[" + StringUtil.trimHandle(componentGbo) + "]耗用数量差[" + needQty + "]");
                }
            }
        }
    }

    public void assembly(String site, String BomComponentRef, String operation, Inventory inventory, String sfcBo, String resource, BigDecimal costQty, String stepId, String shopOrderBo) throws Exception {
        //装配
        AssemblyServiceInterface assemblyServiceInterface = MEServices.create("com.sap.me.production", "AssemblyService", site);
        AssembleComponentsRequest assembleComponentsRequest = new AssembleComponentsRequest();
        List<AssemblyComponent> assemblyComponentList = new ArrayList<>();
        String itemBo = inventory.getItemBo();
        AssemblyComponent assemblyComponent = new AssemblyComponent();
        assemblyComponent.setBomComponentRef(BomComponentRef);
        assemblyComponent.setTimeBased(false);
        assemblyComponent.setQty(costQty);
        assemblyComponent.setActualComponentRef(itemBo);
        List<AssemblyDataField> assemblyDataFieldList = new ArrayList<>();
        AssemblyDataField assemblyDataField = new AssemblyDataField();
        assemblyDataField.setAttribute("INVENTORY_ID_SFC");
        assemblyDataField.setValue(inventory.getInventoryId());
        assemblyDataFieldList.add(assemblyDataField);

        assemblyComponent.setAssemblyDataFields(assemblyDataFieldList);
        assemblyComponentList.add(assemblyComponent);
        assembleComponentsRequest.setComponentList(assemblyComponentList);
        assembleComponentsRequest.setSfcRef(sfcBo);
        assembleComponentsRequest.setOperationRef(operationService.getCurrentRevisionRef(site, operation).getHandle());
        assembleComponentsRequest.setShopOrderRef(shopOrderBo);
        assembleComponentsRequest.setResourceRef(HandleEnum.RESOURCE.getHandle(site, resource));
        assembleComponentsRequest.setRouterRef(routerService.getRouterBySfcBo(sfcBo).getHandle());
        assembleComponentsRequest.setStepId(stepId);
        assembleComponentsRequest.setQuantity(costQty);
        assembleComponentsRequest.setEvent("baseFinished:AssemblyPoint");
        assemblyServiceInterface.assembleByComponents(assembleComponentsRequest);
    }

    @Override
    public void sendErp(List<Map<String,Object>> postMapList)  {
        //请求参数
        ZprodordconfStruIn[] ins = new ZprodordconfStruIn[1];
        for (int i=0;i<postMapList.size();i++){
            Map<String, Object> map = postMapList.get(i);
            String sfc = (String)map.get("sfc");
            Sfc sfcServiceById = sfcService.getById(HandleEnum.SFC.getHandle(CommonMethods.getSite(),sfc));
            String shopOrderBo = sfcServiceById.getShopOrderBo();
            String shopOrder = StringUtil.trimHandle(shopOrderBo);
            String stepId = (String)map.get("stepId");
            BigDecimal qty = (BigDecimal) map.get("qty");
            BigDecimal scrapQty = (BigDecimal)map.get("scrapQty");
            BigDecimal workHour = (BigDecimal)map.get("workHour");
            ZprodordconfStruIn struIn = new ZprodordconfStruIn();
            //生产订单
            struIn.setAufnr(shopOrder);
            //步骤标示
            struIn.setVornr(stepId);
            struIn.setAueru("1");
            //良品数量
            struIn.setLmnga(qty);
            //报废数量
            struIn.setXmnga(scrapQty);
            //机器工时
            struIn.setIsm01(workHour);
            //人工工时
            struIn.setIsm02(workHour);
            struIn.setIsm03(BigDecimal.ZERO);
            struIn.setIsm04(BigDecimal.ZERO);
            struIn.setIsm05(BigDecimal.ZERO);
            struIn.setIsm06(BigDecimal.ZERO);
            ins[i] = struIn;
        }

        TableOfZprodordconfStruInHolder inHolder = new TableOfZprodordconfStruInHolder(ins);

        //返回对象
        ZprodordconfStruOut[] outs = new ZprodordconfStruOut[1];

        ZprodordconfStruOut struOut = new ZprodordconfStruOut();
        struOut.setAufnr("");
        struOut.setVornr("");
        struOut.setAueru("1");
        struOut.setLmnga(new BigDecimal(1));
        struOut.setRet("");
        struOut.setMsg("");
        outs[0] = struOut;
        TableOfZprodordconfStruOutHolder outHolder = new TableOfZprodordconfStruOutHolder(outs);

        //调用WS
        try {
            ERPAPI.erpWebService().zmesProdordconf(inHolder, outHolder);
        } catch (RemoteException e) {
            String status = outHolder.value[1].getRet();
            String message = outHolder.value[1].getMsg();
            if (status.equals("E")) {
                throw new BaseException("ERP接口返回：状态:" + status + ",消息:" + message);
            }
        } finally {
            //记录接口日志
            IntegrationLog log = new IntegrationLog();
            log.setHandle(UUID.randomUUID().toString());
            log.setSite(CommonMethods.getSite());
            log.setIntegrationType(IntegrationTypeConstant.REPORT_ORDER);
            log.setCategory("REQUEST");
            log.setIntegrationWay("ERP");
            log.setIntegrationMethod("erpWebService.zmesProdordconf");
            log.setParam(ins.toString());
            log.setStatus(outHolder.value[1].getRet());
            log.setResultMessage(outHolder.value[1].getMsg());
            log.setTransactionId("");
            log.setRequestDateTime(LocalDateTime.now());
            log.setCreatedDateTime(LocalDateTime.now());
            integrationLogService.saveIntegrationLog(log);
        }
    }

    @Override
    public SfcDto getInfoBySfc(Sfc sfc) {
        String site = CommonMethods.getSite();
        sfc.setSite(site);
        return sfcCrossMapper.getInfoBySfc(sfc);
    }
}
