package com.foreverwin.mesnac.production.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.common.enums.DispatchStatusEnum;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.service.SfcDispatchCommonService;
import com.foreverwin.mesnac.common.util.ExceptionUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.model.Operation;
import com.foreverwin.mesnac.meapi.model.Router;
import com.foreverwin.mesnac.meapi.service.OperationService;
import com.foreverwin.mesnac.meapi.service.RouterService;
import com.foreverwin.mesnac.production.dto.WorkShopTransferDto;
import com.foreverwin.mesnac.production.mapper.SfcCrossMapper;
import com.foreverwin.mesnac.production.mapper.WorkShopTransferMapper;
import com.foreverwin.mesnac.production.model.StepOperation;
import com.foreverwin.mesnac.production.model.WorkShopTransfer;
import com.foreverwin.mesnac.production.service.SfcCrossService;
import com.foreverwin.mesnac.production.service.WorkShopTransferService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 车间转移记录 服务实现类
 * </p>
 *
 * @author Philip
 * @since 2021-07-30
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class WorkShopTransferServiceImpl extends ServiceImpl<WorkShopTransferMapper, WorkShopTransfer> implements WorkShopTransferService {

    @Autowired
    private SfcDispatchCommonService sfcDispatchCommonService;
    @Autowired
    private WorkShopTransferMapper workShopTransferMapper;
    @Autowired
    private OperationService operationService;
    @Autowired
    private SfcCrossService sfcCrossService;
    @Autowired
    private SfcCrossMapper sfcCrossMapper;
    @Autowired
    private RouterService routerService;

    @Override
    public IPage<WorkShopTransfer> selectPage(FrontPage<WorkShopTransfer> frontPage, WorkShopTransfer workShopTransfer) {
        QueryWrapper<WorkShopTransfer> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workShopTransfer);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<WorkShopTransfer> selectList(WorkShopTransfer workShopTransfer) {
        QueryWrapper<WorkShopTransfer> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workShopTransfer);
        return super.list(queryWrapper);
    }

    @Override
    public WorkShopTransferDto sfcEnter(WorkShopTransfer workShopTransfer) {
        //条码是否在工作中心下
        String site = CommonMethods.getSite();
        String sfc = workShopTransfer.getSfc();
        String workCenter = workShopTransfer.getWorkCenter();
        if (StringUtil.isBlank(sfc)) {
            throw new BaseException("产品条码不能为空");
        }

        WorkShopTransferDto dto = workShopTransferMapper.getSfcData(site, sfc);
        if (dto == null) {
            throw new BaseException("未找到该产品条码");
        }
        if (StringUtil.isBlank(dto.getWorkCenter()) || !dto.getWorkCenter().equals(workCenter)) {
            throw new BaseException("产品条码不在所选车间");
        }
        //查询上一工序
        Router router = routerService.getRouterBySfcBo(HandleEnum.SFC.getHandle(site, sfc));
        int seq = 0;
        List<StepOperation> stepOperationList = sfcCrossMapper.findRouterOperationByRouterBo(site, router.getHandle(), LocaleContextHolder.getLocale().getLanguage());
        for (int i = 0; i < stepOperationList.size(); i++) {
            StepOperation stepOperation = stepOperationList.get(i);
            if (stepOperation.getStepId().equals(dto.getStepId())) {
                seq = i;
            }
        }
        if (seq != 0) {
            StepOperation stepOperation = stepOperationList.get(seq - 1);
            dto.setLastOperation(stepOperation.getOperation());
            dto.setLastOperationDesc(stepOperation.getDescription());
        }
        return dto;
    }

    @Override
    public Object transfer(WorkShopTransfer workShopTransfer) {
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        WorkShopTransferDto dto = sfcEnter(workShopTransfer);
        String isOutSource = dto.getIsOutSource();
        String sfcBO = HandleEnum.SFC.getHandle(site, workShopTransfer.getSfc());
        if (StringUtil.notBlank(isOutSource) && isOutSource.equals("Y")) {
            //转出开始
            try {
                Operation currentRevisionRef = operationService.getCurrentRevisionRef(site, dto.getOperation());
                if (workShopTransfer.getType().equals("OUT")) {
                    sfcCrossService.startAction(site, currentRevisionRef.getHandle(), dto.getResrce(), sfcBO, dto.getQty());
                    //更改派工单状态
                    sfcDispatchCommonService.updateSfcDispatchStatus(site, CommonMethods.getUser(), dto.getDispatchNo(), DispatchStatusEnum.START.getCode());
                } else {
                    //转入完成
                    sfcCrossService.completeAction(site, currentRevisionRef.getHandle(),  dto.getResrce(), sfcBO, dto.getQty());
                    //更改派工单状态
                    sfcDispatchCommonService.updateSfcDispatchStatus(site, CommonMethods.getUser(), dto.getDispatchNo(), DispatchStatusEnum.COMPLETE.getCode());
                }
            } catch (Exception e) {
                ExceptionUtil.throwException(e);
            }
        }

        WorkShopTransfer transferLog = new WorkShopTransfer();
        transferLog.setHandle(UUID.randomUUID().toString());
        transferLog.setSite(site);
        transferLog.setWorkCenter(dto.getWorkCenter());
        transferLog.setSfc(dto.getSfc());
        transferLog.setShopOrder(dto.getShopOrder());
        transferLog.setItem(dto.getItem());
        transferLog.setLastOperation(dto.getLastOperation());
        transferLog.setStepId(dto.getStepId());
        transferLog.setOperation(dto.getOperation());
        transferLog.setQty(dto.getQty());
        transferLog.setType(workShopTransfer.getType());
        transferLog.setIsOutSource(isOutSource);
        transferLog.setComments(workShopTransfer.getComments());
        transferLog.setCreateUser(user);
        transferLog.setCreatedDateTime(LocalDateTime.now());
        save(transferLog);
        return null;
    }


}